/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dgws;

import dk.sosi.seal.model.Request;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import java.util.UUID;

public class GenericDGWSHeaderProvider<S, H> implements DGWSHeaderProvider<S, H> {

    private final DGWSRequestFactory dgwsRequestFactory;
    private final Class<S> securityClass;
    private final Class<H> headerClass;
    private final Unmarshaller unmarshaller;

    public GenericDGWSHeaderProvider(DGWSRequestFactory dgwsRequestFactory, Class<S> securityClass, Class<H> headerClass) throws JAXBException {
        this.dgwsRequestFactory = dgwsRequestFactory;
        this.securityClass = securityClass;
        this.headerClass = headerClass;
        this.unmarshaller = JAXBContext.newInstance(securityClass, headerClass).createUnmarshaller();
    }

    @Override
    public DGWSHeaders<S, H> getHeaders() {
        Request request = dgwsRequestFactory.createNewRequest(UUID.randomUUID().toString());
        Document document = request.serialize2DOMDocument();
        document.getDocumentElement().getFirstChild().getChildNodes();
        NodeList headers = document.getDocumentElement().getFirstChild().getChildNodes();

        S security = null;
        H header = null;

        for (int i = 0; i < headers.getLength(); i++) {
            Node element = headers.item(i);
            if (element.getLocalName().equalsIgnoreCase("Security")) {
                security = unmarshal(element, securityClass);
            } else if (element.getLocalName().equalsIgnoreCase("Header")) {
                header = unmarshal(element, headerClass);
            }
        }

        return new DGWSHeaders<>(security, header);
    }

    private <T> T unmarshal(Node element, Class<T> declaredType) {
        try {
            return unmarshaller.unmarshal(element, declaredType).getValue();
        } catch (JAXBException e) {
            return null;
        }
    }


}
