/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dgws.http;

import org.apache.http.client.methods.HttpPost;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.util.Date;

public class HttpClientServiceImpl implements HttpClientService {

    private static final Logger log = Logger.getLogger(HttpClientServiceImpl.class);
    private final HttpExecutor httpExecutor;

    public HttpClientServiceImpl(HttpExecutor httpExecutor) {
        this.httpExecutor = httpExecutor;
        // To avoid CPR logging, prevent debug from loggers below
        LogManager.getLogger("httpclient").setLevel(Level.INFO);
        LogManager.getLogger("org.apache.xml.security.utils").setLevel(Level.INFO);
        LogManager.getLogger("org.apache.http").setLevel(Level.INFO);
    }

    @Override
    public String soapRequest(String endpoint, String soapAction, String requestXml) {
        if (log.isDebugEnabled())
            log.debug(String.format("Executing SOAP request, action=%s, endpoint=%s, request_time=%s", soapAction, endpoint, new Date()));

        HttpPost httpPost = HttpUtil.createSoapRequest(endpoint, soapAction, requestXml);

        return httpExecutor.post(httpPost, httpResponse -> {
            HttpUtil.checkSoapResponseStatus(endpoint, soapAction, httpResponse);
            return HttpUtil.extractResponse(httpResponse);
        });
    }
}
