/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.kafka.serialization;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectReader;
import dk.sundhedsdatastyrelsen.cbs.exceptions.CbsKafkaException;
import org.apache.kafka.common.serialization.Deserializer;

import java.util.Map;

public class JacksonDeserializer<T> implements Deserializer<T> {
    private ObjectReader objectReader;
    private Class<? extends T> targetClass;

    public JacksonDeserializer(Class<? extends T> targetClass) {
        this.targetClass = targetClass;
    }

    @SuppressWarnings("unchecked")
    @Override
    public void configure(Map<String, ?> props, boolean isKey) {
        targetClass = (Class<T>) props.get("JsonPOJOClass");
        if (targetClass == null) {
            throw new CbsKafkaException(JacksonDeserializer.class.getSimpleName() + " not configured with a type to deserialize into");
        }
    }

    @Override
    public T deserialize(String topic, byte[] bytes) {
        if (targetClass == null) {
            throw new CbsKafkaException(JacksonDeserializer.class.getSimpleName() + " not configured with a type to deserialize into");
        }
        if (objectReader == null)
            objectReader = new ObjectMapper().findAndRegisterModules().readerFor(targetClass);

        if (bytes == null)
            return null;

        T data;
        try {
            data = objectReader.readValue(bytes);
        } catch (Exception e) {
            throw new CbsKafkaException(e);
        }

        return data;
    }

    @Override
    public void close() {
        // nothing to do
    }
}
