/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.kafka.config;

import dk.sundhedsdatastyrelsen.cbs.kafka.CbsKafkaConsumer;
import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import dk.sundhedsdatastyrelsen.cbs.test.TestConfiguration;
import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import org.junit.jupiter.api.Test;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;

import static org.junit.jupiter.api.Assertions.*;

@CbsTest
@SpringJUnitConfig(TestConfiguration.class)
class CbsKafkaConfigTest {
    private final CbsKafkaConfig cbsKafkaConfig = new CbsKafkaConfig();

    @Test
    void test_serialization_config() {
        assertNotNull(cbsKafkaConfig.dataFetcherJobSerializer());
        assertNotNull(cbsKafkaConfig.dataFetcherJobDeserializer());
        assertNotNull(cbsKafkaConfig.stringDeserializer());
        assertNotNull(cbsKafkaConfig.stringSerializer());
    }

    @Test
    void test_pool_config() {
        GenericObjectPoolConfig<CbsKafkaConsumer> poolConfig = cbsKafkaConfig.poolConfig(1, 2);
        assertEquals(1, poolConfig.getMaxTotal());
        assertEquals(2, poolConfig.getMaxWaitMillis());
        assertFalse(poolConfig.getLifo());
        assertTrue(poolConfig.getFairness());
        assertTrue(poolConfig.getBlockWhenExhausted());
    }

}
