/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.kafka.serialization;

import com.google.common.collect.ImmutableMap;
import dk.sundhedsdatastyrelsen.cbs.dto.impl.kafka.DataFetcherJobBuilder;
import dk.sundhedsdatastyrelsen.cbs.dto.kafka.DataFetcherJob;
import dk.sundhedsdatastyrelsen.cbs.dto.kafka.FetchType;
import dk.sundhedsdatastyrelsen.cbs.kafka.config.CbsKafkaConfig;
import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import dk.sundhedsdatastyrelsen.cbs.test.KafkaTestUtils;
import dk.sundhedsdatastyrelsen.cbs.test.TestUtils;
import org.apache.kafka.common.serialization.Serializer;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;

import java.util.UUID;

import static com.google.common.truth.Truth.assertThat;

@CbsTest
@SpringJUnitConfig(CbsKafkaConfig.class)
class JacksonSerializerTest {
    @Autowired
    private Serializer<DataFetcherJob> serializer;

    @Test
    void test_serialization_with_single_cpr() {
        String cpr = TestUtils.randomCprNumber();
        String jobId = UUID.randomUUID().toString();
        String messageId = "MyMessageId";
        FetchType fetchType = FetchType.FETCH_ALL;
        DataFetcherJob job = DataFetcherJobBuilder.aDataFetcherJob()
            .withCpr(cpr)
            .withMsgId(messageId)
            .withJobId(jobId)
            .withFetchCprs(ImmutableMap.of(cpr, fetchType))
            .build();

        byte[] bytes = serializer.serialize("unused", job);
        String json = new String(bytes);
        assertThat(json).contains(cpr);
        assertThat(json).contains(jobId);
        assertThat(json).contains(messageId);
        assertThat(json).contains(fetchType.toString());
    }


    @Test
    void test_serialization_with_multiple_fetch_cprs() {
        final DataFetcherJob job = KafkaTestUtils.createRandomDataFetcherJobWithMultipleFetchCprs();

        byte[] bytes = serializer.serialize("unused", job);
        String json = new String(bytes);
        assertThat(json).contains(job.getCpr());
        assertThat(json).contains(job.getJobId());
        assertThat(json).contains(job.getMsgId());
        job.getFetchCprs().forEach((cpr, fetchType) -> {
            assertThat(json).contains(cpr);
            assertThat(json).contains(fetchType.toString());
        });
    }
}
