/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.nas.facade;

import javax.xml.ws.wsaddressing.W3CEndpointReference;
import java.util.UUID;


/**
 * Repository for handling state between NAS getMessages Requests
 */
public interface NasSubscriptionRepository {

    /**
     * Gets a reference to a PullPoint for the specified topic
     * <p>
     * Should be idempotent, i. e. a topics maps 1:1 to specific at PullPoint
     *
     * @param topic the topic for the requested PullPoint
     * @return a reference to a PullPoint with a subscription to the specified topic
     */
    W3CEndpointReference getPullPoint(String topic);

    /**
     * Gets the next MessageId to be used for requesting messages from a NAS PullPoint
     * <p>
     * Request to NAS can be replayed by using the same MessageId
     * <p>
     * Should return the same MessageId until 'commit' has been called
     *
     * @param topic the topic for the requested MessageId
     * @return a MessageId to be used for requesting messages from a NAS Pullpoint
     */
    UUID getNextMessageId(String topic);


    /**
     * Used to signal that processing of messages requested with current MessageId has completed successfully
     * <p>
     * Should make 'getNextMessageId' return a new unique MessageId
     *
     * @param topic the topic for which processing was successful
     */
    void commit(String topic);

}
