/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.logging;

import dk.sds.nsp.audit.AuditBuilder;

import java.util.ArrayList;
import java.util.List;

import static dk.sds.nsp.audit.AuditBuilder.TypeOfInformation.NonPersonalInformation;
import static dk.sds.nsp.audit.AuditBuilder.TypeOfInformation.RegularPersonalInformation;
import static dk.sds.nsp.audit.AuditBuilder.TypeOfInformation.SensitivePersonalInformation;

public class AuditData {

    private final List<Entry> entries = new ArrayList<>();

    public AuditData withNonPersonalInformation(String key, Object value) {
        addEntry(NonPersonalInformation, key, value);
        return this;
    }

    public AuditData withRegularPersonalInformation(String key, Object value) {
        addEntry(RegularPersonalInformation, key, value);
        return this;
    }

    public AuditData withSensitivePersonalInformation(String key, Object value) {
        addEntry(SensitivePersonalInformation, key, value);
        return this;
    }

    private void addEntry(AuditBuilder.TypeOfInformation type, String key, Object value) {
        if (key != null && value != null)
            entries.add(new Entry(type, key, value));

    }

    void write(AuditBuilder auditBuilder, String component, String context) {
        entries.forEach(entry -> {
            if (entry.value instanceof Iterable)
                auditBuilder.addAuditInformation(component, context, entry.type, entry.key, (Iterable<?>) entry.value);
            else
                auditBuilder.addAuditInformation(component, context, entry.type, entry.key, entry.value);
        });
    }

    private static class Entry {
        private final AuditBuilder.TypeOfInformation type;
        private final String key;
        private final Object value;

        private Entry(AuditBuilder.TypeOfInformation type, String key, Object value) {
            this.type = type;
            this.key = key;
            this.value = value;
        }
    }
}
