/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.logging;

import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sdsd.nsp.slalog.ws.SLALoggingServletFilter;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

public class EnhancedSlaLoggingServletFilter extends SLALoggingServletFilter {

    private final Map<String, String> defaultSoapActionMap;

    public EnhancedSlaLoggingServletFilter(Map<String, String> defaultSoapActionMap) {
        this.defaultSoapActionMap = defaultSoapActionMap;
    }

    @Override
    protected SLALogItem createLogItem(HttpServletRequest servletRequest) {
        final HttpServletRequest request = wrapRequest(servletRequest);
        return super.createLogItem(request);
    }

    private HttpServletRequest wrapRequest(HttpServletRequest servletRequest) {
        if (defaultSoapActionMap.containsKey(servletRequest.getServletPath())) {
            return new DefaultSOAPActionRequestWrapper(servletRequest, defaultSoapActionMap.get(servletRequest.getServletPath()));
        }

        return servletRequest;
    }
}
