/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.logging;

import dk.sdsd.nsp.slalog.api.SLALogConfig;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import org.apache.log4j.Logger;

/**
 * This SlaLogger is used for SLA logging of outbound calls
 */
public class SlaLogger {
    private static final Logger log = Logger.getLogger(SlaLogger.class);
    private static final Object lock = new Object();
    private static SLALogConfig slaLogInstance;
    private final String componentName;
    private final String name;

    public SlaLogger(String componentName, String name) {
        this.componentName = componentName;
        this.name = name;
        synchronized (lock) {
            if (slaLogInstance == null) {
                slaLogInstance = new SLALogConfig(componentName, componentName);
            }
        }
    }

    public void addSlaLog(SLALogItem mainItem, long startNanos, long startMillis, String errorMessage) {
        if (slaLogInstance != null && mainItem != null) {
            try {
                SLALogItem logItem = slaLogInstance.getSLALogger().createLogItem(componentName + "." + name, componentName + "." + name);

                logItem.setClientIP(mainItem.getClientIP());
                logItem.setSourceSOAPAction(mainItem.getSourceSOAPAction());
                logItem.setSourceOperation(mainItem.getSourceOperation());
                logItem.setSourceEndpoint(mainItem.getSourceEndpoint());
                logItem.setMessageId(mainItem.getMessageId());
                logItem.setStartTime(startNanos, startMillis);
                logItem.setEndTime(System.nanoTime(), System.currentTimeMillis());

                if (errorMessage == null) {
                    logItem.setCallResultOk();
                } else {
                    logItem.setCallResultError(errorMessage);
                }

                logItem.store();
            } catch (Exception ex) {
                log.error("Could not create SLA LogItem", ex);
            }
        }
    }
}
