/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.logging;

import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;

import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import java.util.Collections;
import java.util.HashMap;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

@CbsTest
class EnhancedSlaLoggingServletFilterTest {

    @Test
    void uses_soap_action_map() throws ServletException {
        final HashMap<String, String> defaultSoapActionMap = new HashMap<>();
        defaultSoapActionMap.put("/foo", "my-foo-action");
        final EnhancedSlaLoggingServletFilter filter = new EnhancedSlaLoggingServletFilter(defaultSoapActionMap);
        filter.init(mock(FilterConfig.class));

        HttpServletRequest mockRequest = Mockito.mock(HttpServletRequest.class);
        when(mockRequest.getServletPath()).thenReturn("/foo");

        final SLALogItem logItem = filter.createLogItem(mockRequest);
        assertEquals("my-foo-action", logItem.getSourceSOAPAction());
    }

    @Test
    void empty_action_map() throws ServletException {
        final EnhancedSlaLoggingServletFilter filter = new EnhancedSlaLoggingServletFilter(Collections.emptyMap());
        filter.init(mock(FilterConfig.class));

        HttpServletRequest mockRequest = Mockito.mock(HttpServletRequest.class);
        when(mockRequest.getServletPath()).thenReturn("/foo");
        when(mockRequest.getHeader(eq("SOAPAction"))).thenReturn("unmapped-soap-action");

        final SLALogItem logItem = filter.createLogItem(mockRequest);
        assertEquals("unmapped-soap-action", logItem.getSourceSOAPAction());
    }
}
