/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs;

import org.apache.log4j.MDC;

import java.util.function.Supplier;

public class CbsLogging {
    public static final String TRACE_ID_PROP = "traceId"; //used in log4.xml as 'traceId=%X{traceId}'
    public static final String MESSAGE_ID_PROP = "msgId"; //used in log4.xml as 'msgId=%X{msgId}'
    public static final String JOB_ID_PROP = "jobId"; //used in log4.xml as 'jobId=%X{jobId}'

    private CbsLogging() {
    }

    public static <T> Supplier<T> supplyWithMDCs(String key1, Object val1, String key2, Object val2, String key3, Object val3, Supplier<T> supplier) {
        return () -> withMDCs(key1, val1, key2, val2, key3, val3, supplier);
    }

    public static <T> Supplier<T> supplyWithMDC(String key, Object o, Supplier<T> supplier) {
        return () -> withMDC(key, o, supplier);
    }

    public static void withMDC(String key, Object o, Runnable runnable) {
        withMDC(key, o, () -> {
            runnable.run();
            return null;
        });
    }

    public static void withMDCs(String key1, Object val1, String key2, Object val2, String key3, Object val3, Runnable runnable) {
        withMDCs(key1, val1, key2, val2, key3, val3, () -> {
            runnable.run();
            return null;
        });
    }

    public static <T> T withMDCs(String key1, Object val1, String key2, Object val2, String key3, Object val3, Supplier<T> supplier) {
        try {
            if (key1 != null)
                MDC.put(key1, val1);
            if (key2 != null)
                MDC.put(key2, val2);
            if (key3 != null)
                MDC.put(key3, val3);
            return supplier.get();
        } finally {
            if (key1 != null)
                MDC.remove(key1);
            if (key2 != null)
                MDC.remove(key2);
            if (key3 != null)
                MDC.remove(key3);
        }
    }

    public static <T> T withMDC(String key, Object o, Supplier<T> supplier) {
        try {
            MDC.put(key, o);
            return supplier.get();
        } finally {
            MDC.remove(key);
        }
    }
}
