/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.spring;

import org.springframework.beans.factory.NoUniqueBeanDefinitionException;
import org.springframework.context.ApplicationContext;
import org.springframework.core.ResolvableType;
import org.springframework.lang.NonNull;

public final class Beans {
    private Beans() {}

    @SuppressWarnings("unchecked")
    @NonNull
    public static <T> T get(@NonNull final ApplicationContext context, @NonNull final Class<?> clazz, @NonNull final Class<?>... generics) throws NoUniqueBeanDefinitionException {
        final ResolvableType type = ResolvableType.forClassWithGenerics(clazz, generics);
        final T bean = (T) context.getBeanProvider(type).getIfUnique();
        if (bean != null) return bean;

        final String[] beanNamesFound = context.getBeanNamesForType(type);
        throw new NoUniqueBeanDefinitionException(type, beanNamesFound);
    }

    @NonNull
    public static <T> T get(@NonNull final ApplicationContext context, @NonNull final Class<T> clazz) throws NoUniqueBeanDefinitionException {
        final T bean = context.getBeanProvider(clazz).getIfUnique();
        if (bean != null) return bean;

        final String[] beanNamesFound = context.getBeanNamesForType(clazz);
        throw new NoUniqueBeanDefinitionException(clazz, beanNamesFound);
    }

}
