/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.status;

import java.util.Map;

public class StatusCheck {

    private final String title;
    private final Map<String, String> info;
    private final Status status;

    public StatusCheck(String title, Map<String, String> info, Status status) {
        this.title = title;
        this.info = info;
        this.status = status;
    }

    public String getTitle() {
        return title;
    }

    public Map<String, String> getInfo() {
        return info;
    }

    public Status getStatus() {
        return status;
    }

    public enum Status {
        OK(1, 200), WARN(2, 202), ERROR(3, 500);

        private final int severity;
        private final int httpCode;

        Status(int severity, int httpCode) {
            this.severity = severity;
            this.httpCode = httpCode;
        }

        public int severity() {
            return severity;
        }

        public int httpCode() {
            return httpCode;
        }
    }

}
