/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs;

import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.CsvSource;
import org.junit.jupiter.params.provider.ValueSource;

import static org.junit.jupiter.api.Assertions.*;

@CbsTest
class CPRFilterTest {

    @ParameterizedTest
    @CsvSource({
        "0506504003,<redacted>",
        "050650-4003,<redacted>",
        "this is a string with a cpr 0101010000,this is a string with a cpr <redacted>",
        "this is another cpr -0101010000-,this is another cpr -<redacted>-",
        "this is a string with 4 cpr 0506504001 0506504002 (050650-4003)+0506504004,this is a string with 4 cpr <redacted> <redacted> (<redacted>)+<redacted>",
    })
    void should_replaces_cpr_matches_with_placeholder(String input, String expected) {
        assertEquals(expected, CPRFilter.filter(input));
    }

    @ParameterizedTest
    @ValueSource(strings = {
        "Normal String",
        "this is a phone number 20304050",
        "this is a long number 123456789123456789"
    })
    void should_not_filter_strings_with_no_cpr(String input) {
        assertEquals(input, CPRFilter.filter(input));
    }

}
