/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs;

import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import org.apache.log4j.MDC;
import org.junit.jupiter.api.Test;

import java.util.function.Supplier;

import static org.junit.jupiter.api.Assertions.*;

@CbsTest
class CbsLoggingTest {


    @Test
    void should_have_mdc_in_runnable() {
        assertNull(MDC.get("key"));

        CbsLogging.withMDC("key", "value", () -> assertEquals("value", MDC.get("key")));

        assertNull(MDC.get("key"));
    }

    @Test
    void should_have_mdc_in_supplier() {
        assertNull(MDC.get("key"));

        int result = CbsLogging.withMDC("key", "value", () -> {
            assertEquals("value", MDC.get("key"));
            return 1;
        });

        assertEquals(1, result);

        assertNull(MDC.get("key"));
    }

    @Test
    void should_have_mdc_in_supplier_when_run() {
        assertNull(MDC.get("key"));

        Supplier<Integer> supplier = CbsLogging.supplyWithMDC("key", "value", () -> {
            assertEquals("value", MDC.get("key"));
            return 1;
        });

        assertEquals(1, supplier.get());

        assertNull(MDC.get("key"));
    }

}
