/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.test;

import org.apache.log4j.Logger;
import org.junit.jupiter.api.extension.BeforeEachCallback;
import org.junit.jupiter.api.extension.ExtensionContext;
import org.junit.jupiter.api.extension.TestWatcher;

import java.util.Optional;

public class TestWatcherLogger implements TestWatcher, BeforeEachCallback {
    private static final Logger logger = Logger.getLogger(TestWatcherLogger.class);

    @Override
    public void testDisabled(ExtensionContext extensionContext, Optional<String> optional) {
        logger.info("Test disabled:   " + extensionContext.getTestClass().orElseThrow(RuntimeException::new).getSimpleName() + "." + extensionContext.getTestMethod().orElseThrow(RuntimeException::new).getName());
    }

    @Override
    public void testSuccessful(ExtensionContext extensionContext) {
        logger.info("Test successful: " + extensionContext.getTestClass().orElseThrow(RuntimeException::new).getSimpleName() + "." + extensionContext.getTestMethod().orElseThrow(RuntimeException::new).getName());
    }

    @Override
    public void testAborted(ExtensionContext extensionContext, Throwable throwable) {
        logger.info("Test aborted:    " + extensionContext.getTestClass().orElseThrow(RuntimeException::new).getSimpleName() + "." + extensionContext.getTestMethod().orElseThrow(RuntimeException::new).getName());
    }

    @Override
    public void testFailed(ExtensionContext extensionContext, Throwable throwable) {
        logger.info("Test failed:     " + extensionContext.getTestClass().orElseThrow(RuntimeException::new).getSimpleName() + "." + extensionContext.getTestMethod().orElseThrow(RuntimeException::new).getName());
    }

    @Override
    public void beforeEach(ExtensionContext extensionContext) throws Exception {
        logger.info("Test started:    " + extensionContext.getTestClass().orElseThrow(RuntimeException::new).getSimpleName() + "." + extensionContext.getTestMethod().orElseThrow(RuntimeException::new).getName());
    }
}
