/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datacache.invalidator.impl;


import dk.sundhedsdatastyrelsen.cbs.audit.SystemAuditLogger;
import dk.sundhedsdatastyrelsen.cbs.dao.LabResultRecord;
import dk.sundhedsdatastyrelsen.cbs.dao.impl.jdbc.LabResultRecordImpl;
import dk.sundhedsdatastyrelsen.cbs.datacache.Writer;
import dk.sundhedsdatastyrelsen.cbs.datacache.invalidator.CacheInvalidationRecord;
import dk.sundhedsdatastyrelsen.cbs.datacache.invalidator.RecordInvalidator;
import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResults;

public class LabResultRecordInvalidator implements RecordInvalidator {

    private final Writer<LabResultRecord<LabResults>> writer;

    public LabResultRecordInvalidator(Writer<LabResultRecord<LabResults>> writer) {
        this.writer = writer;
    }

    @Override
    public boolean invalidate(CacheInvalidationRecord invalidationRecord) {
        LabResultRecordImpl<LabResults> labResultRecord = new LabResultRecordImpl<LabResults>()
            .cpr(invalidationRecord.getCpr())
            .traceId(invalidationRecord.getTraceId())
            .invalidationTime(invalidationRecord.getInvalidationTime());

        SystemAuditLogger systemAuditLogger = SystemAuditLogger.newAudit()
            .addEntry("operation", LabResultRecordInvalidator.class.getName())
            .addEntry("personIdentifier", invalidationRecord.getCpr())
            .addEntry("invalidationTime", invalidationRecord.getInvalidationTime());

        try {
            boolean success = writer.write(labResultRecord);
            systemAuditLogger.addEntry("success", success);
            return success;
        } catch (Exception e) {
            systemAuditLogger
                .addEntry("exception", e.getClass().getName())
                .addEntry("exceptionMessage", e.getMessage());
            throw e;
        } finally {
            systemAuditLogger.log();
        }
    }
}
