/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datacache.invalidator.impl;

import dk.nsi.advis.v10.NotifyContent;
import dk.sundhedsdatastyrelsen.cbs.datacache.invalidator.CacheInvalidationRecord;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.Constants;
import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import dk.vaccinationsregister.schemas._2021._03._01.VaccinationModificationType;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.oasis_open.docs.wsn.b_2.NotificationMessageHolderType;
import org.oasis_open.docs.wsn.b_2.TopicExpressionType;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;

@CbsTest
class VaccinationMessageRecordMapperTest {

    private final org.oasis_open.docs.wsn.b_2.ObjectFactory b2ObjectFactory = new org.oasis_open.docs.wsn.b_2.ObjectFactory();
    private final dk.nsi.advis.v10.ObjectFactory nsiObjectFactory = new dk.nsi.advis.v10.ObjectFactory();
    private final dk.vaccinationsregister.schemas._2021._03._01.ObjectFactory ddvObjectFactory = new dk.vaccinationsregister.schemas._2021._03._01.ObjectFactory();


    private VaccinationMessageRecordMapper vaccinationMessageRecordMapper;

    @BeforeEach
    void setUp() {
        vaccinationMessageRecordMapper = new VaccinationMessageRecordMapper(10L);
    }

    @Test
    void should_map_vaccination_modification_to_record() {
        CacheInvalidationRecord cacheInvalidationRecord = vaccinationMessageRecordMapper.toRecord(getVaccinationModificationMessage("20506504003"));

        assertNotNull(cacheInvalidationRecord);
    }

    @Test
    void should_not_map_non_vaccination_modification_to_record() {
        CacheInvalidationRecord cacheInvalidationRecord = vaccinationMessageRecordMapper.toRecord(getNotificationMessageHolderType("0506504003", "other payload"));

        assertNull(cacheInvalidationRecord);
    }


    private NotificationMessageHolderType getVaccinationModificationMessage(String cpr) {
        VaccinationModificationType vaccinationModificationType = ddvObjectFactory.createVaccinationModificationType();

        return getNotificationMessageHolderType(cpr, vaccinationModificationType);
    }

    private NotificationMessageHolderType getNotificationMessageHolderType(String cpr, Object payload) {
        NotifyContent notifyContent = nsiObjectFactory.createNotifyContent();
        notifyContent.setAny(payload);
        notifyContent.setId(cpr);
        notifyContent.setIdType(Constants.ID_TYPE_CPR);
        notifyContent.setIsSystemNotification(false);

        NotificationMessageHolderType.Message message = b2ObjectFactory.createNotificationMessageHolderTypeMessage();
        message.setAny(notifyContent);

        TopicExpressionType topicExpressionType = b2ObjectFactory.createTopicExpressionType();
        topicExpressionType.setDialect(Constants.TOPIC_EXPRESSION_SIMPLE);
        topicExpressionType.getContent().add("myTopic");

        NotificationMessageHolderType notificationMessageHolderType = b2ObjectFactory.createNotificationMessageHolderType();
        notificationMessageHolderType.setMessage(message);
        notificationMessageHolderType.setTopic(topicExpressionType);
        return notificationMessageHolderType;
    }
}
