/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datacache.invalidator.impl;

import dk.sundhedsdatastyrelsen.cbs.dao.VaccinationRecord;
import dk.sundhedsdatastyrelsen.cbs.datacache.Writer;
import dk.sundhedsdatastyrelsen.cbs.datacache.invalidator.CacheInvalidationRecord;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.Vaccination;
import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.Mock;

import java.time.LocalDateTime;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@CbsTest
class VaccinationRecordInvalidatorTest {

    @Mock
    private Writer<VaccinationRecord<Vaccination>> vaccinationRecordWriter;

    @Captor
    private ArgumentCaptor<VaccinationRecord<Vaccination>> vaccinationRecordArgumentCaptor;

    private VaccinationRecordInvalidator vaccinationRecordInvalidator;

    @BeforeEach
    void setUp() {
        vaccinationRecordInvalidator = new VaccinationRecordInvalidator(vaccinationRecordWriter);
    }

    @Test
    void should_map_and_write_record() {

        String cpr = "cpr";
        String trace = "trace";
        LocalDateTime now = LocalDateTime.now();
        CacheInvalidationRecord cacheInvalidationRecord = new CacheInvalidationRecord(cpr, trace, now);

        vaccinationRecordInvalidator.invalidate(cacheInvalidationRecord);

        verify(vaccinationRecordWriter).write(vaccinationRecordArgumentCaptor.capture());

        VaccinationRecord<Vaccination> captorValue = vaccinationRecordArgumentCaptor.getValue();

        assertAll(
            () -> assertEquals(cpr, captorValue.cpr()),
            () -> assertEquals(trace, captorValue.traceId()),
            () -> assertEquals(now, captorValue.invalidationTime())
        );
    }

    @Test
    void should_return_writers_response() {
        when(vaccinationRecordWriter.write(any())).thenReturn(true, false);

        CacheInvalidationRecord cacheInvalidationRecord = new CacheInvalidationRecord("cpr", "trace", LocalDateTime.now());

        assertTrue(vaccinationRecordInvalidator.invalidate(cacheInvalidationRecord));
        assertFalse(vaccinationRecordInvalidator.invalidate(cacheInvalidationRecord));
    }
}
