/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datacache.invalidator.servlet;

import dk.sundhedsdatastyrelsen.cbs.datacache.invalidator.impl.LockProvider;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.NasConsumer;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;

import static dk.sundhedsdatastyrelsen.cbs.datacache.invalidator.DataCacheInvalidationConfiguration.LAB_RESULT_NAS_CONSUMER_BEAN;
import static dk.sundhedsdatastyrelsen.cbs.datacache.invalidator.DataCacheInvalidationConfiguration.VACCINATION_NAS_CONSUMER_BEAN;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

@Configuration
public class MinimalTestConfig {

    @Primary
    @Bean
    @Qualifier(VACCINATION_NAS_CONSUMER_BEAN)
    public NasConsumer vaccinationNasConsumer() {
        return mock(NasConsumer.class);
    }

    @Primary
    @Bean
    @Qualifier(LAB_RESULT_NAS_CONSUMER_BEAN)
    public NasConsumer labResultNasConsumer() {
        return mock(NasConsumer.class);
    }


    @Primary
    @Bean
    public LockProvider lockProvider() {
        LockProvider mock = mock(LockProvider.class);
        when(mock.acquireLock(any())).thenReturn(true);
        return mock;
    }

}
