/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dao.impl.jdbc;

import dk.sundhedsdatastyrelsen.cbs.dao.EUAntigenSpec;
import org.springframework.lang.NonNull;

import java.time.LocalDateTime;

// EU Antigen test specification (not all AG tests are valid wrt. EU rules)
public class EUAntigenSpecImpl implements EUAntigenSpec {

    static final String ID_COLUMN_LABEL = "PID";
    static final String ID_DEVICE_COLUMN_LABEL = "idDevice";
    static final String VALID_FROM_LABEL = "ValidFrom";
    static final String VALID_TO_LABEL = "ValidTo";

    private long id;
    private String idDevice;
    private LocalDateTime validFrom;
    private LocalDateTime validTo;

    @Override
    @NonNull
    public long id() {
        return id;
    }

    public EUAntigenSpecImpl id(@NonNull final long id) {
        this.id = id;
        return this;
    }

    @NonNull
    @Override
    public String idDevice() {
        return idDevice;
    }

    public EUAntigenSpecImpl idDevice(@NonNull final String idDevice) {
        this.idDevice = idDevice;
        return this;
    }

    public EUAntigenSpecImpl validFrom(@NonNull final LocalDateTime validFrom) {
        this.validFrom = validFrom;
        return this;
    }

    @Override
    public LocalDateTime validFrom() {
        return validFrom;
    }

    public EUAntigenSpecImpl validTo(LocalDateTime validTo) {
        this.validTo = validTo;
        return this;
    }

    @Override
    public LocalDateTime validTo() {
        return validTo;
    }

}
