/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datacache.impl;

import dk.sundhedsdatastyrelsen.fluent.spring.jdbc.KeyMapper;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayNameGeneration;
import org.junit.jupiter.api.DisplayNameGenerator;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Answers;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.jdbc.core.JdbcTemplate;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@ExtendWith(MockitoExtension.class)
@DisplayNameGeneration(DisplayNameGenerator.ReplaceUnderscores.class)
class WriterWithDeleteTest {

    @Mock
    private JdbcTemplate jdbcTemplate;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private KeyMapper<String, String> keyMapper;
    private Writers.WriterWithDeleteImpl<String, String> writerWithDelete;

    @BeforeEach
    void setUp() {
        when(keyMapper.insertTableMetaData().tableName()).thenReturn("table");
        when(keyMapper.keyColumnLabel()).thenReturn("key");
        when(keyMapper.serializeKey("theKey")).thenReturn("theSerializedKey");

        writerWithDelete = new Writers.WriterWithDeleteImpl<>(jdbcTemplate, keyMapper);
    }

    @Test
    void should_execute_delete_sql() {
        writerWithDelete.delete("theKey");

        verify(jdbcTemplate).update("delete from table where key = ?", "theSerializedKey");
    }

    @Test
    void should_return_true_if_affected_rows_is_one() {
        when(jdbcTemplate.update(anyString(), anyString())).thenReturn(1);

        boolean result = writerWithDelete.delete("theKey");

        assertTrue(result);
    }

    @Test
    void should_return_false_if_affected_rows_is_not_one() {
        when(jdbcTemplate.update(anyString(), anyString())).thenReturn(0);

        assertFalse(writerWithDelete.delete("theKey"));

        when(jdbcTemplate.update(anyString(), anyString())).thenReturn(2);

        assertFalse(writerWithDelete.delete("theKey"));
    }


}
