# Data Cache

The data cache rests upon a MariaDb database. This is started and deployed as part of docker-compose. In order to create
the tables and also to regulate versions and changes, we are using [Liquibase](https://docs.liquibase.com/home.html)

## Usage

There are a few ways to kick-off a Liquibase changeset sequence. The approach currently used is the bean found
in `PersistenceConfiguration::liquibase` during application startup from `ServiceServletContextListener`

Liquibase can also be run manually in isolation using:

``` bash
mvn liquibase:update
```

To get the full list of Liquibase goals:

``` bash
  mvn liquibase:help
```

## Change Sets

Changes to the database are managed through the `changelog-master.xml` file. This file references the sql files found
in `src/main/resources/db/changelog/changesets` and executes them in the order found i.e. top to bottom. This can be
used as a guide for adding further change sets. Available options such as `runAlways` or `failOnError`can be found in
the [Liquibase ChangeSets](https://docs.liquibase.com/concepts/basic/changeset.html) documentation.

``` xml
<changeSet id="sensible-id-filename" author="mbl@nine.dk">
  <preConditions onFail="MARK_RAN">
    <changeSetExecuted id="initial-database-setup" author="cdj@trifork.com"/>
  </preConditions>
  <comment>Description of what the changeSet does</comment>
  <sqlFile path="src/main/resources/db/changelog/changesets/sensible-id-filename.sql"/>
</changeSet>
```

Liquibase automatically creates a `DATABASECHANGELOG` table in the database. During execution `DATABASECHANGELOG` is
checked for each change set entry in `changelog-master.xml`, to see if the combination of *id / author / filepath* has
been run.

If it has been run, the change set will be skipped unless there is a runAlways tag set to true in the change set
element. After all the changes have run, Liquibase will insert a new row with the id/author/filepath along with a *
MD5Sum* of the changeset in the `DATABASECHANGELOG`.
