/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datacache.configuration;

import dk.sundhedsdatastyrelsen.cbs.datacache.liquibase.AsyncLiquibaseService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.task.SimpleAsyncTaskExecutor;
import org.springframework.jdbc.datasource.lookup.JndiDataSourceLookup;
import org.springframework.scheduling.annotation.EnableAsync;

import javax.sql.DataSource;

@EnableAsync
@Configuration
public class PersistenceConfiguration {

    @Value("${datacache.datasource.jndi-name:java:jboss/datasources/cbs}")
    private String dataSourceJndiName;

    @Value("${liquibase.datasource.jndi-name:java:jboss/datasources/liquibase}")
    private String liquibaseDataSourceJndiName;

    /**
     * JDBC datasource configuration
     *
     * @return DataSource
     */
    @Bean
    public DataSource dataSource() {
        return new JndiDataSourceLookup().getDataSource(dataSourceJndiName);
    }

    /**
     * Asynchronous service that executes the liquibase changelog without blocking application startup
     *
     * @return AsyncLiquibaseService
     */
    @Bean
    public AsyncLiquibaseService asyncLiquibaseService() {
        final DataSource dataSource = new JndiDataSourceLookup().getDataSource(liquibaseDataSourceJndiName);
        return new AsyncLiquibaseService(dataSource);
    }

    @Bean
    public SimpleAsyncTaskExecutor simpleAsyncTaskExecutor() {
        return new SimpleAsyncTaskExecutor();
    }
}
