/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datacache.status;

import dk.sundhedsdatastyrelsen.cbs.status.StatusCheck;
import dk.sundhedsdatastyrelsen.cbs.status.StatusChecker;
import org.springframework.jdbc.core.JdbcTemplate;

import java.util.HashMap;
import java.util.Map;

public class JdbcStatusChecker implements StatusChecker {

    private final JdbcTemplate jdbcTemplate;
    private final String title;

    public JdbcStatusChecker(JdbcTemplate jdbcTemplate, String title) {
        this.jdbcTemplate = jdbcTemplate;
        this.title = title;
    }

    @Override
    public StatusCheck checkStatus() {
        boolean isOk = false;
        final String sql = "select 1";

        final Map<String, String> info = new HashMap<>();

        try {
            jdbcTemplate.execute(sql);
            isOk = true;
        } catch (Exception e) {
            info.put("Error", e.getMessage());
        }
        info.put("Connected", String.valueOf(isOk));
        info.put("Test Query", sql);

        return new StatusCheck(title, info, isOk ? StatusCheck.Status.OK : StatusCheck.Status.ERROR);
    }

}
