/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datacache.test;

import dk.sundhedsdatastyrelsen.cbs.datacache.liquibase.AsyncLiquibaseService;
import org.junit.jupiter.api.extension.BeforeAllCallback;
import org.junit.jupiter.api.extension.ExtensionContext;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.datasource.DriverManagerDataSource;
import org.springframework.jdbc.datasource.lookup.JndiDataSourceLookup;
import org.springframework.jndi.JndiTemplate;
import org.springframework.mock.jndi.SimpleNamingContextBuilder;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.util.AopTestUtils;

public class InMemoryDbExtension implements BeforeAllCallback {

    @Override
    public void beforeAll(ExtensionContext context) throws Exception {
        //noinspection deprecation
        SimpleNamingContextBuilder.emptyActivatedContextBuilder();
        final JndiTemplate jndiTemplate = new JndiDataSourceLookup().getJndiTemplate();
        final DriverManagerDataSource dataSource = new DriverManagerDataSource("jdbc:h2:mem:cbs-datacache;MODE=MYSQL;DB_CLOSE_DELAY=-1");
        jndiTemplate.bind("java:jboss/datasources/euantigenspec", dataSource);
        jndiTemplate.bind("java:jboss/datasources/cbs", dataSource);
        jndiTemplate.bind("java:jboss/datasources/kids", dataSource);
        jndiTemplate.bind("java:jboss/datasources/liquibase", dataSource);

        final ApplicationContext applicationContext = SpringExtension.getApplicationContext(context);
        AsyncLiquibaseService liquibaseService = applicationContext.getBean(AsyncLiquibaseService.class);
        // unwrap bean to run synchronously
        AopTestUtils.<AsyncLiquibaseService>getUltimateTargetObject(liquibaseService).update();
    }
}
