-- liquibase formatted sql
-- changeset Trifork:test-database-setup

create or replace table vaccination
(
    cpr               char(10)                               not null,
    trace_id          varchar(50)                            not null,
    creation_time     timestamp(6) default current_timestamp not null,
    invalidation_time timestamp                              null,
    data              longtext                               null,
    constraint pk__vaccination primary key (cpr, creation_time),
    constraint chk__vaccination check (not (data is null and invalidation_time is null))
) engine = innodb;

create or replace view vaccination_view as
select v.cpr,
       v.trace_id,
       v.creation_time,
       v.invalidation_time,
       v.data
from (select non_expired_writes.cpr, max(non_expired_writes.creation_time) max_creation_time
      from vaccination non_expired_writes
               left join vaccination active_invalidations on
              non_expired_writes.cpr = active_invalidations.cpr and
              active_invalidations.data is null and
              active_invalidations.invalidation_time <= current_timestamp and
              active_invalidations.invalidation_time >= non_expired_writes.creation_time
      where non_expired_writes.data is not null
        and (non_expired_writes.invalidation_time > current_timestamp or non_expired_writes.invalidation_time is null)
        and active_invalidations.cpr is null
      group by non_expired_writes.cpr) s
         join vaccination v on v.cpr = s.cpr and v.creation_time = s.max_creation_time;

create or replace table lab_result
(
    cpr               char(10)                               not null,
    trace_id          varchar(50)                            not null,
    creation_time     timestamp(6) default current_timestamp not null,
    invalidation_time timestamp                              null,
    data              longtext                               null,
    constraint pk__lab_result primary key (cpr, creation_time),
    constraint chk__lab_result check (not (data is null and invalidation_time is null))
)
    engine = innodb;

create or replace view lab_result_view as
select l.cpr,
       l.trace_id,
       l.creation_time,
       l.invalidation_time,
       l.data
from (select non_expired_writes.cpr, max(non_expired_writes.creation_time) max_creation_time
      from lab_result non_expired_writes
               left join lab_result active_invalidations on
              non_expired_writes.cpr = active_invalidations.cpr and
              active_invalidations.data is null and
              active_invalidations.invalidation_time <= current_timestamp and
              active_invalidations.invalidation_time >= non_expired_writes.creation_time
      where non_expired_writes.data is not null
        and (non_expired_writes.invalidation_time > current_timestamp or non_expired_writes.invalidation_time is null)
        and active_invalidations.cpr is null
      group by non_expired_writes.cpr) s
         join lab_result l on l.cpr = s.cpr and l.creation_time = s.max_creation_time;

create or replace table stamdata_person
(
    person_identifier char(10)    not null,
    givenName         varchar(34) not null,
    familyName        varchar(34) not null,
    PRIMARY KEY (person_identifier)
) engine = innodb;

create or replace table stamdata_child_relation
(
    person_identifier char(10) not null,
    child_identifier  char(10) not null,
    PRIMARY KEY (person_identifier, child_identifier)
) engine = innodb;

create or replace table nas_lock
(
    topic        varchar(255) not null,
    locked_until timestamp    not null,
    constraint pk__nas_lock primary key (topic)
) engine = innodb;

create or replace table nas_status
(
    topic      varchar(255) not null,
    message_id binary(16)   not null,
    constraint pk__nas_status primary key (topic)
) engine = innodb;

create or replace table nas_subscription
(
    topic        varchar(255) not null,
    pull_point   varchar(255) not null,
    subscription varchar(255) not null,
    constraint pk__nas_sub primary key (topic)
) engine = innodb;

create or replace table test_result
(
    test_identifier            bigint primary key not null,
    person_identifier          char(10)           not null,
    type_of_test               varchar(15)        not null,
    test_name                  varchar(100)       not null,
    test_manufacturer          varchar(100)       not null,
    timestamp_sample           timestamp          not null,
    timestamp_result           timestamp          not null,
    result_of_the_test         varchar(15)        not null,
    testing_centre_or_facility varchar(100)       not null,
    timestamp_last_update      timestamp          not null,
    sletteflag                 tinyint            not null
);

create or replace index ix__test_result_cpr on test_result (person_identifier);
create or replace index ix__test_result_delete on test_result (timestamp_result, result_of_the_test);

create or replace view test_result_view as
select test_identifier            id,
       person_identifier          cpr,
       type_of_test               type,
       test_name                  name,
       test_manufacturer          manufacturer,
       timestamp_sample           sample_timestamp,
       timestamp_result           result_timestamp,
       result_of_the_test         result,
       testing_centre_or_facility center,
       timestamp_last_update      creation_timestamp
from test_result
where not sletteflag;

CREATE TABLE IF NOT EXISTS AGTI_invitrodiagnosticdevices
(
    PID                   BIGINT(15) AUTO_INCREMENT NOT NULL PRIMARY KEY,
    idDevice              VARCHAR(16),
    commercialName        VARCHAR(500),
    idManufacturer        VARCHAR(16),
    manufacturerName      VARCHAR(500),
    manufacturerCountry   VARCHAR(50),
    manufacturerWebsite   VARCHAR(500),
    ceMarking             BOOLEAN,
    hscCommonList         BOOLEAN,
    hscMutualRecognition  BOOLEAN,
    formats               VARCHAR(500),
    physicalSupports      VARCHAR(500),
    targets               VARCHAR(500),
    specimens             VARCHAR(500),
    pathogens             VARCHAR(500),
    lineages              VARCHAR(500),
    commercialStatus      VARCHAR(50),
    lastUpdated           VARCHAR(50),
    idTest                VARCHAR(16),
    assayCategory         VARCHAR(500),
    rapidDiagnostic       BOOLEAN,
    selfTest              BOOLEAN,
    readerRequired        BOOLEAN,
    method                VARCHAR(500),
    measurement           VARCHAR(500),
    subclass              VARCHAR(500),
    detectionPrinciple    VARCHAR(500),
    subcategory           VARCHAR(500),
    timeMinutes           VARCHAR(50),
    lod                   VARCHAR(500),
    positiveControl       VARCHAR(500),
    negativeControl       VARCHAR(500),
    performanceControl    VARCHAR(500),
    calibration           VARCHAR(500),
    analyticalSensitivity VARCHAR(500),
    analyticalSpecificity VARCHAR(500),
    crossreactivity       VARCHAR(500),
    fp                    VARCHAR(500),
    fn                    VARCHAR(500),
    precisionX            VARCHAR(500),
    accuracy              VARCHAR(500),
    reproducibility       VARCHAR(500),
    robustness            VARCHAR(500),
    clinicalSensitivity   VARCHAR(500),
    clinicalSpecificity   VARCHAR(500),
    throughput            VARCHAR(500),
    typeOfAntigen         VARCHAR(500),
    ValidFrom             DateTime                  NOT NULL,
    ValidTo               DateTime,
    ModifiedDate          DateTime                  NOT NULL,
    DataSetUUID           varchar(40),
    KEY `IdxId` (`idDevice` ASC),
    KEY `currentidx` (`idDevice` ASC, `ValidTo` ASC, `ValidFrom` ASC),
    KEY `krsidx` (`ModifiedDate` ASC, `PID` ASC),
    KEY `validix` (`ValidTo` ASC, `ValidFrom` ASC)
);

CREATE VIEW AGTI_antigentestregister_cbs_view
AS
SELECT *
FROM AGTI_invitrodiagnosticdevices ag
WHERE hscCommonList = 1;
