/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datafetcher.ddv;

import dk.sdsd.dgws._2010._08.NameFormat;
import dk.sdsd.dgws._2010._08.OrgUsingID;
import dk.sdsd.dgws._2012._06.WhiteListingHeader;

public class WhitelistingConfig {

    final String systemName;

    final String systemOwnerName;

    final String systemVersion;

    final String orgUsingIdCVR;

    final String orgUsingIdName;

    final String requestedRole;

    public WhitelistingConfig(String systemName, String systemOwnerName, String systemVersion, String orgUsingIdCVR, String orgUsingIdName, String requestedRole) {
        this.systemName = systemName;
        this.systemOwnerName = systemOwnerName;
        this.systemVersion = systemVersion;
        this.orgUsingIdCVR = orgUsingIdCVR;
        this.orgUsingIdName = orgUsingIdName;
        this.requestedRole = requestedRole;
    }

    WhiteListingHeader createWhiteListingHeader() {
        WhiteListingHeader h = new WhiteListingHeader();
        h.setSystemName(systemName);
        h.setSystemOwnerName(systemOwnerName);
        h.setSystemVersion(systemVersion);
        OrgUsingID orgUsingID = new OrgUsingID();
        orgUsingID.setValue(orgUsingIdCVR);
        orgUsingID.setNameFormat(NameFormat.MEDCOM_CVRNUMBER);
        h.setOrgUsingID(orgUsingID);
        h.setOrgUsingName(orgUsingIdName);
        h.setRequestedRole(requestedRole);
        return h;
    }
}
