/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datafetcher.model;

import dk.sundhedsdatastyrelsen.cbs.dto.kafka.DataFetcherJob;
import org.apache.commons.lang3.StringUtils;

import java.util.stream.Collectors;
import java.util.stream.Stream;

public final class DataFetcherJobExecutionResultBuilder {
    private DataFetcherJob dataFetcherJob;
    private boolean failed;
    private String failureMessage;

    private DataFetcherJobExecutionResultBuilder() {
    }

    public static DataFetcherJobExecutionResultBuilder aDataFetcherJobExecutionResult() {
        return new DataFetcherJobExecutionResultBuilder();
    }

    public DataFetcherJobExecutionResultBuilder withDataFetcherJob(DataFetcherJob dataFetcherJob) {
        this.dataFetcherJob = dataFetcherJob;
        return this;
    }

    public DataFetcherJobExecutionResultBuilder withFailed(boolean failed) {
        this.failed = failed;
        return this;
    }

    public DataFetcherJobExecutionResultBuilder withFailureMessage(String failureMessage) {
        this.failureMessage = failureMessage;
        return this;
    }

    public DataFetcherJobExecutionResult build() {
        return new DataFetcherJobExecutionResult(dataFetcherJob, failed, failureMessage);
    }


    public DataFetcherJobExecutionResultBuilder append(DataFetcherJobExecutionResult other) {
        if (this.dataFetcherJob == null) {
            throw new IllegalStateException("Cannot append execution to null DataFetcherJob");
        }
        if (other.getDataFetcherJob() != this.dataFetcherJob) {
            throw new IllegalArgumentException("Cannot append execution result from other DataFetcherJob");
        }

        this.failed = this.failed || other.isFailed();
        if (other.isFailed()) {
            this.failureMessage = Stream.of(this.failureMessage, other.getFailureMessage())
                .filter(StringUtils::isNotEmpty)
                .collect(Collectors.joining(" AND "));
        }
        return this;
    }
}
