/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datafetcher.servlet;

import dk.sundhedsdatastyrelsen.cbs.datacache.liquibase.AsyncLiquibaseService;
import dk.sundhedsdatastyrelsen.cbs.datacache.status.JdbcStatusChecker;
import dk.sundhedsdatastyrelsen.cbs.datacache.test.InMemoryDbExtension;
import dk.sundhedsdatastyrelsen.cbs.datafetcher.DataFetcher;
import dk.sundhedsdatastyrelsen.cbs.kafka.status.KafkaStatusChecker;
import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Answers;
import org.mockito.Mock;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.servlet.ServletContextEvent;

import static dk.sundhedsdatastyrelsen.cbs.datafetcher.servlet.DataFetcherServletContextListener.SPRING_APPLICATION_CONTEXT;
import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.*;

@CbsTest
@ExtendWith({InMemoryDbExtension.class})
class DataFetcherServletContextListenerTest {
    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private ServletContextEvent servletContextEvent;

    @Configuration
    public static class MockConfig {
        @Bean
        KafkaStatusChecker kafkaStatusChecker() {
            return mock(KafkaStatusChecker.class);
        }

        @Bean
        JdbcStatusChecker jdbcStatusChecker() {
            return mock(JdbcStatusChecker.class);
        }

        @Bean
        public AsyncLiquibaseService asyncLiquibaseService() {
            return mock(AsyncLiquibaseService.class);
        }
    }

    @Test
    void test_app_lifecycle() {
        DataFetcherServletContextListener contextListener = new DataFetcherServletContextListener();
        assertDoesNotThrow(() -> contextListener.contextInitialized(servletContextEvent));
        verify(servletContextEvent.getServletContext()).setAttribute(eq(SPRING_APPLICATION_CONTEXT), any(AnnotationConfigApplicationContext.class));

        AnnotationConfigApplicationContext applicationContext = mock(AnnotationConfigApplicationContext.class);
        DataFetcher dataFetcher = mock(DataFetcher.class);
        when(servletContextEvent.getServletContext().getAttribute(SPRING_APPLICATION_CONTEXT)).thenReturn(applicationContext);
        when(applicationContext.getBean(DataFetcher.class)).thenReturn(dataFetcher);

        assertDoesNotThrow(() -> contextListener.contextDestroyed(servletContextEvent));
        verify(dataFetcher).close();
        verify(applicationContext).close();
    }
}
