/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.ddv;

import dk.sundhedsdatastyrelsen.cbs.datafetcher.config.DDVConfig;
import dk.sundhedsdatastyrelsen.cbs.datafetcher.ddv.DDVVaccinePasService;
import dk.sundhedsdatastyrelsen.cbs.datafetcher.exceptions.DDVException;
import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import dk.vaccinationsregister.schemas._2013._12._01.GetVaccinationPassportAsPDFResponseType;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;

import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;

@CbsTest
@SpringJUnitConfig({DDVConfig.class, DdvTestConfiguration.class})
class DDVVaccinePasServiceClientTest {

    @Autowired
    DDVVaccinePasService ws;

    @Test
    void testDDSClient() throws Exception {
        String cpr = "2401010014";
        GetVaccinationPassportAsPDFResponseType vp = ws.getVaccinationPassport(cpr, UUID.randomUUID().toString());
        assertEquals(cpr, vp.getPersonInfo().getPersonCivilRegistrationIdentifier());
    }

    @Test
    void testNotFound() {
        String cpr = "2401019014";
        final String jobId = UUID.randomUUID().toString();
        Assertions.assertThrows(DDVException.class, () -> ws.getVaccinationPassport(cpr, jobId));
    }

}
