/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.ddv;

import dk.sundhedsdatastyrelsen.cbs.dgws.http.HttpClientService;
import org.apache.commons.io.FileUtils;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;

import java.io.File;

@Configuration
public class DdvTestConfiguration {

    @Primary
    @Bean
    public HttpClientService httpClientService() {
        return (endpoint, soapAction, requestXml) -> {
            if ("\"Issue\"".equals(soapAction)) {
                return getFile("sts-response.xml");
            } else {
                if (requestXml.contains("2401010010")) { return getFile("2401010010_ok_pfizer.xml");  }
                if (requestXml.contains("2401010011")) { return getFile("2401010011_notok_one_vaccination.xml");  }
                if (requestXml.contains("2401010012")) { return getFile("2401010012_ok_moderna.xml");  }
                if (requestXml.contains("2401010014")) { return getFile("2401010014_notok_min-interval-violated.xml");  }
                if (requestXml.contains("2401010018")) { return getFile("2401010018_ok_az-and-moderna.xml");  }
                if (requestXml.contains("2401019014")) { return getFile("2401019014-not-found.xml");  }
                return getFile("1111111118.xml");
            }
        };
    }

    public String getFile(String filename) {
        try {
            return FileUtils.readFileToString(new File(getClass().getClassLoader().getResource("ddv-responses/" + filename).getFile()), "UTF-8");
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }


}
