/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.ddv.converter;

import dk.oio.rep.cpr_dk.xml.schemas.core._2006._01._17.SimpleCPRPersonType;
import dk.sundhedsdatastyrelsen.cbs.test.RandomData;
import dk.vaccinationsregister.schemas._2013._12._01.ATCType;
import dk.vaccinationsregister.schemas._2013._12._01.EffectuationOverviewType;
import dk.vaccinationsregister.schemas._2013._12._01.GetVaccinationPassportAsPDFResponseType;
import dk.vaccinationsregister.schemas._2013._12._01.VaccinationStatus;
import dk.vaccinationsregister.schemas._2013._12._01.VaccineType;
import org.springframework.lang.NonNull;

public final class DDVTestUtils {

    private static final RandomData RANDOM_DATA = new RandomData("CBS".hashCode());

    private DDVTestUtils() {
    }

    @NonNull
    static GetVaccinationPassportAsPDFResponseType newDdvResponse(@NonNull final String cpr) {
        final GetVaccinationPassportAsPDFResponseType card = new GetVaccinationPassportAsPDFResponseType();
        card.setPersonInfo(new SimpleCPRPersonType());
        int pickRandomEnumValue = RANDOM_DATA.nextInt(VaccinationStatus.values().length);
        card.setVaccinationStatus(VaccinationStatus.values()[pickRandomEnumValue]);
        card.setVaccinationAgainst("SARS-CoV-2 (COVID-19)");
        card.getPersonInfo().setPersonCivilRegistrationIdentifier(cpr);
        card.getEffectuation().addAll(RANDOM_DATA.list(10, DDVTestUtils::newEffectuation));
        card.getAdditionalEffectuation().addAll(RANDOM_DATA.list(2, DDVTestUtils::newEffectuation));
        return card;
    }

    @NonNull
    static EffectuationOverviewType newEffectuation() {
        final EffectuationOverviewType effectuation = new EffectuationOverviewType();
        VaccineType vaccineType = new VaccineType();
        vaccineType.setVaccineIdentifier(RANDOM_DATA.nextLong());
        ATCType atcType = new ATCType();
        atcType.setCode(RANDOM_DATA.string(RANDOM_DATA.nextInt(0, 5)));
        atcType.setText(RANDOM_DATA.string(RANDOM_DATA.nextInt(0, 5)));
        vaccineType.setATC(atcType);
        effectuation.setVaccine(vaccineType);
        effectuation.setDrugName(RANDOM_DATA.string(RANDOM_DATA.nextInt(0, 5)));
        effectuation.setEffectuatedDateTime(RANDOM_DATA.zonedDateTime());
        effectuation.setCountryIdentificationCode(RANDOM_DATA.countryCode());
        return effectuation;
    }
}
