/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.impl.lab;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.core.TreeNode;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.deser.std.StdDeserializer;
import org.springframework.lang.NonNull;

import java.io.IOException;

import static com.fasterxml.jackson.core.JsonToken.VALUE_TRUE;
import static com.fasterxml.jackson.databind.JsonMappingException.from;
import static dk.sundhedsdatastyrelsen.cbs.dto.impl.lab.LabResultImpl.POSITIVE_PROPERTY;
import static java.lang.String.format;
import static java.util.Optional.ofNullable;

public class AntigenResultDeserializer extends StdDeserializer<LabResultImpl.AntigenResultImpl> {

    public static final String COULD_NOT_FIND_PROPERTY = "Could not find property %s";

    protected AntigenResultDeserializer() {
        super(LabResultImpl.AntigenResultImpl.class);
    }

    @Override
    @NonNull
    public LabResultImpl.AntigenResultImpl deserialize(
        @NonNull final JsonParser parser,
        @NonNull final DeserializationContext context) throws IOException {
        final ObjectCodec codec = parser.getCodec();
        final TreeNode treeNode = codec.readTree(parser);

        final boolean positive = ofNullable(treeNode.path(POSITIVE_PROPERTY))
            .map(TreeNode::asToken)
            .map(VALUE_TRUE::equals)
            .orElseThrow(() -> from(parser, format(COULD_NOT_FIND_PROPERTY, POSITIVE_PROPERTY)));

        return positive
            ? codec.treeToValue(treeNode, LabResultImpl.AntigenResultImpl.PositiveAntigenResultImpl.class)
            : codec.treeToValue(treeNode, LabResultImpl.AntigenResultImpl.NegativeAntigenResultImpl.class);
    }
}
