/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.impl.lab;

import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResult;
import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResults;
import org.apache.commons.lang3.NotImplementedException;
import org.springframework.lang.NonNull;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import static java.lang.String.format;

public class LabResultsBuilder {

    @NonNull private final String cpr;
    @NonNull private final List<LabResult.PcrResult> pcrResults;
    @NonNull private final List<LabResult.AntigenResult> antigenResults;

    private LabResultsBuilder(@NonNull final String cpr) {
        this.cpr = cpr;
        pcrResults = new ArrayList<>();
        antigenResults = new ArrayList<>();
    }

    @NonNull
    public static LabResultsBuilder cpr(@NonNull final String cpr) {
        return new LabResultsBuilder(cpr);
    }

    @NonNull
    public LabResultsBuilder add(@NonNull final LabResult.PcrResult result) {
        pcrResults.add(result);
        return this;
    }

    @NonNull
    public LabResultsBuilder add(@NonNull final LabResult.AntigenResult result) {
        antigenResults.add(result);
        return this;
    }

    @NonNull
    public LabResultsBuilder add(@NonNull final LabResult result) {
        if (result instanceof LabResult.PcrResult) return add((LabResult.PcrResult) result);
        if (result instanceof LabResult.AntigenResult) return add((LabResult.AntigenResult) result);

        throw new NotImplementedException(
            format("No handler for %s class %s implemented", LabResult.class, result.getClass().getName()));
    }

    @NonNull
    public final LabResults build() {
        return new LabResultsImpl(cpr, pcrResults, antigenResults);
    }

    public LabResultsBuilder addAll(Collection<LabResult> results) {
        results.stream().forEach(this::add);
        return this;
    }
}
