/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import dk.sundhedsdatastyrelsen.cbs.dto.impl.config.DtoImplConfiguration;
import dk.sundhedsdatastyrelsen.cbs.dto.impl.lab.LabResultBuilder;
import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResult;
import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResult.AntigenResult;
import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResult.PcrResult;
import dk.sundhedsdatastyrelsen.cbs.test.RandomData;
import org.springframework.lang.NonNull;

import static org.junit.jupiter.api.Assertions.assertEquals;

public final class Harness {
    @NonNull public static final RandomData RANDOM_DATA = new RandomData("CBS".hashCode());
    @NonNull public static final ObjectMapper OBJECT_MAPPER = new DtoImplConfiguration().objectMapper();

    @NonNull
    public static LabResult newLabResult() {
        return RANDOM_DATA.nextBoolean() ? newPcrResult() : newAntigenResult();
    }

    @NonNull
    public static PcrResult newPcrResult() {
        return LabResultBuilder.pcrResult(
            RANDOM_DATA.nextLong(),
            RANDOM_DATA.localDateTime(),
            RANDOM_DATA.localDateTime())
            .center(RANDOM_DATA.nextBoolean() ? RANDOM_DATA.string(10) : null)
            .manufacturer(RANDOM_DATA.nextBoolean() ? RANDOM_DATA.string(10) : null)
            .name(RANDOM_DATA.nextBoolean() ? RANDOM_DATA.string(10) : null)
            .positive(RANDOM_DATA.nextBoolean())
            .build();
    }

    @NonNull
    public static AntigenResult newAntigenResult() {
        return LabResultBuilder.antigenResult(
            RANDOM_DATA.nextLong(),
            RANDOM_DATA.localDateTime(),
            RANDOM_DATA.localDateTime())
            .center(RANDOM_DATA.nextBoolean() ? RANDOM_DATA.string(10) : null)
            .manufacturer(RANDOM_DATA.nextBoolean() ? RANDOM_DATA.string(10) : null)
            .positive(RANDOM_DATA.nextBoolean())
            .build();
    }

    public static void equals(LabResult source, LabResult deserialized) {
        assertEquals(source.center(), deserialized.center());
        assertEquals(source.id(), deserialized.id());
        assertEquals(source.manufacturer(), deserialized.manufacturer());
        assertEquals(source.name(), deserialized.name());
        assertEquals(source.negative(), deserialized.negative());
        assertEquals(source.positive(), deserialized.positive());
        assertEquals(source.resultDateTime(), deserialized.resultDateTime());
        assertEquals(source.samplingDateTime(), deserialized.samplingDateTime());
        assertEquals(source.type(), deserialized.type());
        assertEquals(source.getClass(), deserialized.getClass());
    }
}
