/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.impl.kafka;

import com.google.common.collect.ImmutableMap;
import com.google.common.truth.Truth;
import dk.sundhedsdatastyrelsen.cbs.dto.kafka.DataFetcherJob;
import dk.sundhedsdatastyrelsen.cbs.dto.kafka.FetchType;
import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import org.junit.jupiter.api.Test;

import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;

@CbsTest
class DataFetcherJobBuilderTest {

    @Test
    void test_builder() {
        final String cpr1 = "0101011111";
        final String cpr2 = "0101012222";
        final String cpr3 = "0101013333";
        final String msgId = UUID.randomUUID().toString();
        final String jobId = UUID.randomUUID().toString();
        final FetchType fetchType1 = FetchType.FETCH_ALL;
        final FetchType fetchType2 = FetchType.FETCH_VACCINATION_STATUS;
        final FetchType fetchType3 = FetchType.FETCH_TEST_STATUS;
        ImmutableMap<String, FetchType> fetchCps = ImmutableMap.<String, FetchType>builder()
            .put(cpr1, fetchType1)
            .put(cpr2, fetchType2)
            .put(cpr3, fetchType3)
            .build();

        final DataFetcherJob built = DataFetcherJobBuilder.aDataFetcherJob()
            .withCpr(cpr1)
            .withMsgId(msgId)
            .withJobId(jobId)
            .withFetchCprs(fetchCps)
            .build();

        assertEquals(cpr1, built.getCpr());
        assertEquals(msgId, built.getMsgId());
        assertEquals(jobId, built.getJobId());
        Truth.assertThat(fetchCps).containsExactly(cpr1, fetchType1, cpr2, fetchType2, cpr3, fetchType3);
    }
}
