/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.impl.vaccination;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.EffectuatedVaccination;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.lang.NonNull;

import java.time.LocalDateTime;

import static com.google.common.truth.Truth.assertThat;
import static dk.sundhedsdatastyrelsen.cbs.dto.impl.Harness.OBJECT_MAPPER;
import static dk.sundhedsdatastyrelsen.cbs.dto.impl.Harness.RANDOM_DATA;
import static dk.sundhedsdatastyrelsen.cbs.dto.impl.vaccination.EffectuatedVaccinationBuilder.effectuatedVaccination;
import static java.time.LocalDateTime.now;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assumptions.assumeTrue;

class EffectuatedVaccinationImplTest {

    @Test
    void serializeDeserialize() throws JsonProcessingException {
        final LocalDateTime effectuated = now();
        final EffectuatedVaccination source = effectuatedVaccination(2L, effectuated, "GL").build();

        final String serialized = OBJECT_MAPPER.writeValueAsString(source);
        assertThat(serialized).contains("\"countryCode\":\"GL\"");

        final EffectuatedVaccination deserialized = OBJECT_MAPPER.readValue(serialized, EffectuatedVaccination.class);
        assertNotNull(deserialized);
        assertEquals(effectuated, deserialized.effectuated());
        assertEquals(2L, deserialized.drugIdentifier());
        assertEquals("GL", deserialized.countryCode());
        assertEquals(source.getClass(), deserialized.getClass());
    }

    @Test
    @DisplayName("Unknown property in serialized form")
    void unknownProperty() throws JsonProcessingException {
        final EffectuatedVaccination source = new UnknownPropertyEffectuatedVaccination(effectuatedVaccination(2L, now(), RANDOM_DATA.countryCode()).build());

        final String serialized = OBJECT_MAPPER.writeValueAsString(source);
        assertNotNull(serialized);
        assumeTrue(serialized.contains(UnknownPropertyEffectuatedVaccination.UNKNOWN));

        final EffectuatedVaccination deserialized = OBJECT_MAPPER.readValue(serialized, EffectuatedVaccination.class);
        assertNotNull(deserialized);
    }

    private static final class UnknownPropertyEffectuatedVaccination extends EffectuatedVaccinationImpl {
        private static final String UNKNOWN = "unknown";
        @SuppressWarnings({"unused", "FieldCanBeLocal"})
        @JsonProperty("unknown")
        @NonNull
        private final String unknown;

        private UnknownPropertyEffectuatedVaccination(@NonNull final EffectuatedVaccination vaccination) {
            super(
                vaccination.drugIdentifier(),
                vaccination.effectuated(),
                vaccination.countryCode());
            unknown = UNKNOWN;
        }
    }


}
