/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.test;


import com.google.common.collect.ImmutableMap;
import dk.sundhedsdatastyrelsen.cbs.dto.impl.kafka.DataFetcherJobBuilder;
import dk.sundhedsdatastyrelsen.cbs.dto.kafka.DataFetcherJob;
import dk.sundhedsdatastyrelsen.cbs.dto.kafka.FetchType;
import org.springframework.lang.NonNull;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

public class KafkaTestUtils {
    private static final RandomData RANDOM = new RandomData(new Random().nextInt());

    private KafkaTestUtils() {
    }

    public static DataFetcherJob createRandomDataFetcherJobWithMultipleFetchCprs() {
        final String cpr = RANDOM.cpr();
        return createRandomDataFetcherJobWithMultipleFetchCprs(cpr);
    }

    public static DataFetcherJob createRandomDataFetcherJobWithMultipleFetchCprs(String cpr) {
        final ImmutableMap<String, FetchType> fetchCprs = getMultipleRandomFetchCprs(cpr);
        return DataFetcherJobBuilder.aDataFetcherJob()
            .withCpr(cpr)
            .withMsgId(UUID.randomUUID().toString())
            .withJobId(UUID.randomUUID().toString())
            .withFetchCprs(fetchCprs)
            .build();
    }

    @NonNull
    public static ImmutableMap<String, FetchType> getMultipleRandomFetchCprs(String primaryCpr) {
        return getRandomFetchCprs(primaryCpr, 1, 10);
    }

    @NonNull
    public static ImmutableMap<String, FetchType> getRandomFetchCprs(String primaryCpr, int lowerBound, int upperBound) {
        final Map<String, FetchType> fetchCprs = new HashMap<>(RANDOM.map(RANDOM.nextInt(lowerBound, upperBound), () -> new AbstractMap.SimpleEntry<>(RANDOM.cpr(), getRandomFetchType())));
        fetchCprs.put(primaryCpr, getRandomFetchType());
        return ImmutableMap.copyOf(fetchCprs);
    }

    public static DataFetcherJob createRandomDataFetcherJobWithSingleFetchCpr() {
        return createRandomDataFetcherJobWithSingleFetchCpr(getRandomFetchType());
    }

    public static DataFetcherJob createRandomDataFetcherJobWithSingleFetchCpr(FetchType fetchType) {
        final String cpr = RANDOM.cpr();
        return DataFetcherJobBuilder.aDataFetcherJob()
            .withCpr(cpr)
            .withMsgId(UUID.randomUUID().toString())
            .withJobId(UUID.randomUUID().toString())
            .withFetchCprs(ImmutableMap.of(cpr, fetchType))
            .build();
    }

    public static FetchType getRandomFetchType() {
        int pickRandomEnum = RANDOM.nextInt(FetchType.values().length);
        return FetchType.values()[pickRandomEnum];
    }

    public static Set<DataFetcherJob> createRandomJobsWithMultipleFetches(int jobCount) {
        return IntStream.range(0, jobCount).mapToObj(i -> createRandomDataFetcherJobWithMultipleFetchCprs()).collect(Collectors.toCollection(HashSet::new));
    }

    public static Set<DataFetcherJob> createRandomJobsWithSingleFetches(int jobCount) {
        return IntStream.range(0, jobCount).mapToObj(i -> createRandomDataFetcherJobWithSingleFetchCpr()).collect(Collectors.toCollection(HashSet::new));
    }

}
