/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.test;

import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResult;
import dk.sundhedsdatastyrelsen.cbs.test.Data.Result;
import dk.sundhedsdatastyrelsen.cbs.test.Data.Results;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.springframework.lang.NonNull;

import java.time.LocalDateTime;
import java.util.stream.Stream;

import static java.time.LocalDateTime.now;
import static java.util.Objects.requireNonNull;
import static java.util.Optional.of;
import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
import static org.junit.jupiter.api.Assertions.assertEquals;

class ResultTest {
    private static final RandomData RANDOM_DATA = new RandomData();
    private static final int MAX_TEST = 100;

    @ParameterizedTest
    @MethodSource("source")
    void positiveAntigen(@NonNull final LocalDateTime dateTime) {
        assertEquals(
            dateTime,
            of(Result.positiveAntigen(dateTime))
                .map(LabResult::samplingDateTime)
                .orElse(null));
    }

    @ParameterizedTest
    @MethodSource("source")
    void positiveEuUnapprovedAntigen(@NonNull final LocalDateTime dateTime) {
        assertEquals(
            dateTime,
            of(Result.positiveEuUnapprovedAntigen(dateTime))
                .map(LabResult::samplingDateTime)
                .orElse(null));
    }

    @ParameterizedTest
    @MethodSource("source")
    void positivePcr(@NonNull final LocalDateTime dateTime) {
        assertEquals(
            dateTime,
            of(Result.positivePcr(dateTime))
                .map(LabResult::samplingDateTime)
                .orElse(null));
    }

    @ParameterizedTest
    @MethodSource("source")
    void negativeAntigen(@NonNull final LocalDateTime dateTime) {
        assertEquals(
            dateTime,
            of(Result.negativeAntigen(dateTime))
                .map(LabResult::samplingDateTime)
                .orElse(null));
    }

    @ParameterizedTest
    @MethodSource("source")
    void negativeEuUnapprovedAntigen(@NonNull final LocalDateTime dateTime) {
        assertEquals(
            dateTime,
            of(Result.negativeEuUnapprovedAntigen(dateTime))
                .map(LabResult::samplingDateTime)
                .orElse(null));
    }

    @ParameterizedTest
    @MethodSource("source")
    void negativePcr(@NonNull final LocalDateTime dateTime) {
        assertEquals(
            dateTime,
            of(Result.negativePcr(dateTime))
                .map(LabResult::samplingDateTime)
                .orElse(null));
    }


    @NonNull
    private static Stream<Arguments> source() {
        return Stream.generate(RANDOM_DATA::localDateTime).limit(MAX_TEST).map(Arguments::arguments);
    }

    @Test
    @DisplayName("friendly(LabResult)")
    void friendly() {
        assertDoesNotThrow(() -> Data.friendly(requireNonNull(Results.positivePcr(now()).mostRecentResult())));
    }
}
