/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.test;

import dk.sundhedsdatastyrelsen.cbs.test.Data.Vaccinations;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.springframework.lang.NonNull;

import java.time.LocalDateTime;
import java.util.stream.Stream;

import static dk.sundhedsdatastyrelsen.cbs.test.Data.Vaccinations.EMPTY;
import static java.time.LocalDateTime.now;
import static org.junit.jupiter.api.Assertions.*;

class VaccinationsTest {

    private static final RandomData RANDOM_DATA = new RandomData();
    private static final int MAX_TEST = 100;

    @Test
    @SuppressWarnings("ExcessiveLambdaUsage")
    void empty() {
        assertDoesNotThrow(() -> EMPTY);
    }

    @ParameterizedTest
    @MethodSource("source")
    void completed(@NonNull final LocalDateTime dateTime, final boolean multiDose, @NonNull final LocalDateTime... revaccinationDates) {
        assertEquals(
            dateTime,
            multiDose
                ? Vaccinations.completedMultiDose(dateTime).completed()
                : Vaccinations.completedSingleDose(dateTime).completed());

        assertEquals(
            dateTime,
            multiDose
                ? Vaccinations.completedMultiDoseWithAdditionalVaccinations(dateTime, revaccinationDates).completed()
                : Vaccinations.completedSingleDoseWithAdditionalVaccinations(dateTime, revaccinationDates).completed());

        assertEquals(
            dateTime,
            multiDose
                ? Vaccinations.completedMultiDoseWithAdditionalVaccinations(dateTime, revaccinationDates.length).completed()
                : Vaccinations.completedSingleDoseWithAdditionalVaccinations(dateTime, revaccinationDates.length).completed());

        //specific cpr
        final String cpr = RANDOM_DATA.cpr();
        assertEquals(
            dateTime,
            multiDose
                ? Vaccinations.completedMultiDose(cpr, dateTime).completed()
                : Vaccinations.completedSingleDose(cpr, dateTime).completed());

        assertEquals(
            dateTime,
            multiDose
                ? Vaccinations.completedMultiDoseWithAdditionalVaccinations(cpr, dateTime, revaccinationDates).completed()
                : Vaccinations.completedSingleDoseWithAdditionalVaccinations(cpr, dateTime, revaccinationDates).completed());

        assertEquals(
            dateTime,
            multiDose
                ? Vaccinations.completedMultiDoseWithAdditionalVaccinations(cpr, dateTime, revaccinationDates.length).completed()
                : Vaccinations.completedSingleDoseWithAdditionalVaccinations(cpr, dateTime, revaccinationDates.length).completed());
    }

    @ParameterizedTest
    @MethodSource("source")
    void partial(@NonNull final LocalDateTime dateTime, final boolean multiDose) {
        assertNull(Vaccinations.partial(dateTime).completed());
    }

    @NonNull
    private static Stream<Arguments> source() {
        return Stream
            .generate(RANDOM_DATA::localDateTime)
            .limit(MAX_TEST)
            .map(d -> Arguments.arguments(d, RANDOM_DATA.nextBoolean(), RANDOM_DATA.localDateTimes(3).toArray(LocalDateTime[]::new)));
    }

    @Test
    @DisplayName("friendly(Vaccination)")
    void friendly() {
        assertDoesNotThrow(() -> Data.friendly(Vaccinations.completedMultiDose(now())));
    }


}
