/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.kafka;

import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

import java.util.NoSuchElementException;
import java.util.Objects;

import static java.util.Arrays.stream;

public enum FetchType {

    FETCH_VACCINATION_STATUS(0b01),
    FETCH_TEST_STATUS(0b10),
    FETCH_ALL(0b11);

    private final int bitmask;

    FetchType(final int bitmask) {
        this.bitmask = bitmask;
    }

    @Nullable
    public static FetchType of(@NonNull final FetchType... types) throws NoSuchElementException {
        final int mask = stream(types)
            .filter(Objects::nonNull)
            .reduce(0b00, (m, j) -> m | j.bitmask, (a, b) -> a | b);
        return stream(values())
            .filter(j -> j.bitmask == mask)
            .findFirst()
            .orElse(null);
    }
}
