/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.lab;

import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

import java.util.List;

@SuppressWarnings("unused")
public interface LabResults {

    /**
     * Gets the cpr number of the person this lab results instance belongs to
     *
     * @return the cpr number
     */
    @NonNull
    String cpr();

    /**
     * Gets the most recent {@link LabResult} instance <em>irrespective of type and outcome</em>.
     *
     * @return the result, <code>null</code> if no {@link LabResult} instances exist.
     */
    @Nullable
    LabResult mostRecentResult();

    /**
     * Gets the most recent {@link LabResult} instance that is positive <em>irrespective of type</em>.
     *
     * @return the result, <code>null</code> if no positive {@link LabResult} instances exist.
     */
    @Nullable
    LabResult.PositiveResult mostRecentPositiveResult();

    /**
     * Gets the most recent {@link LabResult} instance that is negative <em>irrespective of type</em>.
     *
     * @return the result, <code>null</code> if no negative {@link LabResult} instances exist.
     */
    @Nullable
    LabResult.NegativeResult mostRecentNegativeResult();

    /**
     * Gets the most recent {@link LabResult.PcrResult} instance.
     *
     * @return the result, <code>null</code> if no {@link LabResult.PcrResult} instances exist.
     */
    @Nullable
    LabResult.PcrResult mostRecentPcrResult();

    /**
     * Gets the most recent {@link LabResult.PcrResult} instance that is positive.
     *
     * @return the result, <code>null</code> if no positive {@link LabResult.PcrResult} instances exist.
     */
    @Nullable
    LabResult.PositivePcrResult mostRecentPositivePcrResult();

    /**
     * Get the most recent {@link LabResult.PcrResult} instance that is negative.
     *
     * @return the result, <code>null</code> if no negative {@link LabResult.PcrResult} instances exist.
     */
    @Nullable
    LabResult.NegativePcrResult mostRecentNegativePcrResult();

    /**
     * Gets the most recent {@link LabResult.AntigenResult} instance.
     *
     * @return the result, <code>null</code> if no {@link LabResult.AntigenResult} instances exist.
     */
    @Nullable
    LabResult.AntigenResult mostRecentAntigenResult();

    /**
     * Gets the most recent {@link LabResult.AntigenResult} instance that is positive.
     *
     * @return the result, <code>null</code> if no positive {@link LabResult.AntigenResult} instances exist.
     */
    @Nullable
    LabResult.PositiveAntigenResult mostRecentPositiveAntigenResult();

    /**
     * Get the most recent {@link LabResult.AntigenResult} instance that is negative.
     *
     * @return the result, <code>null</code> if no negative {@link LabResult.AntigenResult} instances exist.
     */
    @Nullable
    LabResult.NegativeAntigenResult mostRecentNegativeAntigenResult();

    /**
     * Gets a sorted collection of {@link LabResult.PcrResult} instances ordered by {@link LabResult#samplingDateTime()} descending.
     *
     * @return the set.
     */
    @NonNull
    List<LabResult.PcrResult> pcrResults();

    /**
     * Gets a sorted collection of {@link LabResult.AntigenResult} instances ordered by {@link LabResult#samplingDateTime()} descending.
     *
     * @return the set.
     */
    @NonNull
    List<LabResult.AntigenResult> antigenResults();

    /**
     * Gets a collection set of {@link LabResult} instances <em>irrespective of type</em> ordered by {@link LabResult#samplingDateTime()} ascending.
     *
     * @return the set.
     */
    @NonNull
    List<LabResult> results();
}
