/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.rules;

import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResults;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.Vaccination;
import dk.sundhedsdatastyrelsen.cbs.rules.evaluation.CombinedPassportEvaluator;
import dk.sundhedsdatastyrelsen.cbs.rules.evaluation.PassportEvaluation;
import dk.sundhedsdatastyrelsen.cbs.rules.evaluation.PassportEvaluations;
import dk.sundhedsdatastyrelsen.cbs.rules.evaluation.RecoveryPassportEvaluation;
import org.springframework.lang.NonNull;

import java.time.LocalDateTime;

public class CombinedPassportEvaluatorImpl implements CombinedPassportEvaluator {

    @NonNull private final VaccinationPassportEvaluator vaccinationPassportEvaluator;
    @NonNull private final RevaccinationPassportEvaluator revaccinationPassportEvaluator;
    @NonNull private final TestPassportEvaluator testPassportEvaluator;
    @NonNull private final RecoveryPassportEvaluator recoveryPassportEvaluator;

    public CombinedPassportEvaluatorImpl(
        @NonNull final VaccinationPassportEvaluator vaccinationPassportEvaluator,
        @NonNull final RevaccinationPassportEvaluator revaccinationPassportEvaluator,
        @NonNull final TestPassportEvaluator testPassportEvaluator,
        @NonNull final RecoveryPassportEvaluator recoveryPassportEvaluator) {
        this.vaccinationPassportEvaluator = vaccinationPassportEvaluator;
        this.revaccinationPassportEvaluator = revaccinationPassportEvaluator;
        this.testPassportEvaluator = testPassportEvaluator;
        this.recoveryPassportEvaluator = recoveryPassportEvaluator;
    }

    @Override
    @NonNull
    public PassportEvaluations evaluate(
        @NonNull final Vaccination vaccination,
        @NonNull final LabResults labResults,
        @NonNull final LocalDateTime now) {
        final PassportEvaluation vaccinationPassportEvaluation = vaccinationPassportEvaluator.evaluate(vaccination, labResults, now);
        final PassportEvaluation revaccinationPassportEvaluation = revaccinationPassportEvaluator.evaluate(vaccination, labResults, now);
        final PassportEvaluation testPassportEvaluation = testPassportEvaluator.evaluate(vaccination, labResults, now);
        final RecoveryPassportEvaluation recoveryPassportEvaluation = recoveryPassportEvaluator.evaluate(vaccination, labResults, now);
        return new PassportEvaluationsImpl(vaccinationPassportEvaluation, revaccinationPassportEvaluation, testPassportEvaluation, recoveryPassportEvaluation);
    }
}
