/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.rules;

import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResults;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.Vaccination;
import dk.sundhedsdatastyrelsen.cbs.rules.evaluation.PassportEvaluation;
import org.springframework.lang.NonNull;

import java.time.LocalDateTime;

@FunctionalInterface
public interface PassportEvaluator<T extends PassportEvaluation> {

    /**
     * Evaluates vaccinations and lab results in the context of passport rules.
     * @param vaccination the vaccinations
     * @param labResults the lab results
     * @param now the date and time of <em>now</em> used in the evaluation
     * @return the result of the evaluation.
     */
    @NonNull
    T evaluate(@NonNull final Vaccination vaccination, @NonNull final LabResults labResults, @NonNull final LocalDateTime now);
}
