/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.rules.configuration;

import org.springframework.lang.NonNull;

import java.time.temporal.TemporalAmount;

public class VaccinationPassportConfiguration {

    @NonNull private final TemporalAmountWithTimeInstruction minimumPeriodAfterCompletedSingleVaccinationSchedule;
    @NonNull private final TemporalAmountWithTimeInstruction minimumPeriodAfterCompletedMultiVaccinationSchedule;
    @NonNull private final TemporalAmountWithTimeInstruction maximumPeriodAfterCompletedVaccinationSchedule;
    @NonNull private final TemporalAmountWithTimeInstruction suspensionPeriodAfterCompletedVaccinationSchedule;
    @NonNull private final TemporalAmountWithTimeInstruction minimumPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule;
    @NonNull private final TemporalAmountWithTimeInstruction maximumPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule;
    @NonNull private final TemporalAmountWithTimeInstruction suspensionPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule;
    @NonNull private final TemporalAmountWithTimeInstruction maximumPeriodAfterCompletedVaccinationScheduleUnder18;

    VaccinationPassportConfiguration(
        @NonNull final TemporalAmountWithTimeInstruction minimumPeriodAfterCompletedSingleVaccinationSchedule,
        @NonNull final TemporalAmountWithTimeInstruction minimumPeriodAfterCompletedMultiVaccinationSchedule,
        @NonNull final TemporalAmountWithTimeInstruction maximumPeriodAfterCompletedVaccinationSchedule,
        @NonNull final TemporalAmountWithTimeInstruction suspensionPeriodAfterCompletedVaccinationSchedule,
        @NonNull final TemporalAmountWithTimeInstruction minimumPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule,
        @NonNull final TemporalAmountWithTimeInstruction maximumPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule,
        @NonNull final TemporalAmountWithTimeInstruction suspensionPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule,
        @NonNull TemporalAmountWithTimeInstruction maximumPeriodAfterCompletedVaccinationScheduleUnder18) {
        this.minimumPeriodAfterCompletedSingleVaccinationSchedule = minimumPeriodAfterCompletedSingleVaccinationSchedule;
        this.minimumPeriodAfterCompletedMultiVaccinationSchedule = minimumPeriodAfterCompletedMultiVaccinationSchedule;
        this.maximumPeriodAfterCompletedVaccinationSchedule = maximumPeriodAfterCompletedVaccinationSchedule;
        this.suspensionPeriodAfterCompletedVaccinationSchedule = suspensionPeriodAfterCompletedVaccinationSchedule;
        this.minimumPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule = minimumPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule;
        this.maximumPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule = maximumPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule;
        this.suspensionPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule = suspensionPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule;
        this.maximumPeriodAfterCompletedVaccinationScheduleUnder18 = maximumPeriodAfterCompletedVaccinationScheduleUnder18;
    }

    @NonNull
    public TemporalAmount minimumPeriodAfterCompletedSingleVaccinationSchedule() {
        return minimumPeriodAfterCompletedSingleVaccinationSchedule.amount();
    }

    @NonNull
    public TemporalAmount minimumPeriodAfterCompletedMultiVaccinationSchedule() {
        return minimumPeriodAfterCompletedMultiVaccinationSchedule.amount();
    }

    @NonNull
    public TemporalAmount maximumPeriodAfterCompletedVaccinationSchedule() {
        return maximumPeriodAfterCompletedVaccinationSchedule.amount();
    }

    @NonNull
    public TemporalAmount suspensionPeriodAfterCompletedVaccinationSchedule() {
        return suspensionPeriodAfterCompletedVaccinationSchedule.amount();
    }

    public boolean keepTimeOnMinimumPeriodAfterCompletedSingleVaccinationSchedule() {
        return minimumPeriodAfterCompletedSingleVaccinationSchedule.keepTime();
    }

    public boolean keepTimeOnMinimumPeriodAfterCompletedMultiVaccinationSchedule() {
        return minimumPeriodAfterCompletedMultiVaccinationSchedule.keepTime();
    }

    public boolean keepTimeOnMaximumPeriodAfterCompletedVaccinationSchedule() {
        return maximumPeriodAfterCompletedVaccinationSchedule.keepTime();
    }

    public boolean keepTimeOnSuspensionPeriodAfterCompletedVaccinationSchedule() {
        return suspensionPeriodAfterCompletedVaccinationSchedule.keepTime();
    }
    @NonNull
    public TemporalAmount minimumPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule() {
        return minimumPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule.amount();
    }

    @NonNull
    public TemporalAmount maximumPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule() {
        return maximumPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule.amount();
    }

    @NonNull
    public TemporalAmount suspensionPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule() {
        return suspensionPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule.amount();
    }

    public boolean keepTimeOnMinimumPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule() {
        return minimumPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule.keepTime();
    }

    public boolean keepTimeOnMaximumPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule() {
        return maximumPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule.keepTime();
    }

    public boolean keepTimeOnSuspensionPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule() {
        return suspensionPeriodAfterFirstVaccinationOfPartiallyCompletedVaccinationSchedule.keepTime();
    }

    public TemporalAmount maximumPeriodAfterCompletedVaccinationScheduleUnder18() {
        return maximumPeriodAfterCompletedVaccinationScheduleUnder18.amount();
    }

    public boolean keepTimeOnMaximumPeriodAfterCompletedVaccinationScheduleUnder18() {
        return maximumPeriodAfterCompletedVaccinationScheduleUnder18.keepTime();
    }
}
