/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.rules.evaluation;

import org.springframework.lang.NonNull;

import java.time.LocalDateTime;

class RecoveryLabResultImpl implements RecoveryLabResult {

    @NonNull private final PositivePcrResult result;
    @NonNull private final LocalDateTime recoveryValidFrom;
    @NonNull private final LocalDateTime recoveryValidUntil;

    RecoveryLabResultImpl(
        @NonNull final PositivePcrResult result,
        @NonNull final LocalDateTime recoveryValidFrom,
        @NonNull final LocalDateTime recoveryValidUntil) {

        this.result = result;
        this.recoveryValidFrom = recoveryValidFrom;
        this.recoveryValidUntil = recoveryValidUntil;
    }

    @Override
    public long id() {
        return result.id();
    }

    @Override
    public boolean positive() {
        return result.positive();
    }

    @Override
    public boolean negative() {
        return result.negative();
    }

    @Override
    @NonNull
    public String type() {
        return result.type();
    }

    @Override
    public String name() {
        return result.name();
    }

    @Override
    public String manufacturer() {
        return result.manufacturer();
    }

    @Override
    public String center() {
        return result.center();
    }

    @Override
    @NonNull
    public LocalDateTime samplingDateTime() {
        return result.samplingDateTime();
    }

    @Override
    @NonNull
    public LocalDateTime resultDateTime() {
        return result.resultDateTime();
    }

    @Override
    @NonNull
    public LocalDateTime recoveryValidFrom() {
        return recoveryValidFrom;
    }

    @Override
    @NonNull
    public LocalDateTime recoveryValidUntil() {
        return recoveryValidUntil;
    }
}
