/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.rules.cucumber.expression;

import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

import java.time.LocalDateTime;
import java.time.temporal.TemporalAmount;

import static dk.sundhedsdatastyrelsen.cbs.rules.cucumber.expression.State.initial;
import static java.util.Arrays.stream;

public class ExpressionEvaluator {

    @NonNull private final Resolver<String, LocalDateTime> dateTimeResolver;
    @NonNull private final Resolver<String, TemporalAmount> amountResolver;

    public ExpressionEvaluator(
        @NonNull final Resolver<String, LocalDateTime> dateTimeResolver,
        @NonNull final Resolver<String, TemporalAmount> amountResolver) {
        this.dateTimeResolver = dateTimeResolver;
        this.amountResolver = amountResolver;
    }

    @Nullable
    public LocalDateTime calculate(@NonNull final String expression) {
        if("".equals(expression)) return null;

        final State state = stream(expression.split(" "))
            .sequential()
            .reduce(
                initial(dateTimeResolver, amountResolver),
                State::next,
                (x, y) -> null // never called as stream is consumed sequential
            );
        if(state.valid()) return state.value();

        throw new RuntimeException(new IllegalStateException(state.error()));
    }
}
