/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.rules.harness;

import org.springframework.lang.NonNull;

import java.time.LocalDateTime;
import java.time.temporal.TemporalAmount;

public final class LocalDateTimes {

    private LocalDateTimes() {}

    @NonNull
    public static LocalDateTime beforeMin(@NonNull final LocalDateTime time, @NonNull final TemporalAmount amount, final boolean keepTime) {
        return keepTime ? time.minus(amount).minusNanos(1) : time.minus(amount).minusDays(1);
    }

    @NonNull
    public static LocalDateTime beforeMax(@NonNull final LocalDateTime time, @NonNull final TemporalAmount amount, final boolean keepTime) {
        return keepTime ? time.minus(amount).plusNanos(1) : time.minus(amount).plusDays(1);
    }

    @NonNull
    public static LocalDateTime afterMin(@NonNull final LocalDateTime time, @NonNull final TemporalAmount amount, final boolean keepTime) {
        return keepTime ? time.minus(amount).plusNanos(1) : time.minus(amount).plusDays(1);
    }

    @NonNull
    public static LocalDateTime afterMax(@NonNull final LocalDateTime time, @NonNull final TemporalAmount amount, final boolean keepTime) {
        return keepTime ? time.minus(amount).minusNanos(1) : time.minus(amount).minusDays(1);
    }

    @NonNull
    public static LocalDateTime on(@NonNull final LocalDateTime time, @NonNull final TemporalAmount amount, final boolean keepTime) {
        return keepTime ? time.minus(amount) : time.minus(amount);
    }

    @NonNull
    public static LocalDateTime expireDenied(@NonNull final LocalDateTime time, @NonNull final TemporalAmount amount, final boolean keepTime) {
        return keepTime ? time.plus(amount) : time.plus(amount).plusDays(1).toLocalDate().atStartOfDay();
    }

    @NonNull
    public static LocalDateTime expireGranted(@NonNull final LocalDateTime time, @NonNull final TemporalAmount amount, final boolean keepTime) {
        return keepTime ? time.plus(amount) : time.plus(amount).plusDays(1).toLocalDate().atStartOfDay();
    }
}
