/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.config;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;

import java.time.temporal.TemporalAmount;

@Configuration
public class EUCovidTestCertificateSigningConfiguration {

    @Value("#{T(dk.sundhedsdatastyrelsen.cbs.service.util.TemporalAmountParser).parse('${eu.covid.test.certificate.signing.cwt.test-result.maximum-validity:P7D}')}")
    private TemporalAmount cwtTestResultMaxValidity;

    @Value("#{T(dk.sundhedsdatastyrelsen.cbs.service.util.TemporalAmountParser).parse('${eu.covid.test.certificate.signing.cwt.recovery.maximum-validity:P180D}')}")
    private TemporalAmount cwtRecoveryMaxValidity;

    @Value("#{T(dk.sundhedsdatastyrelsen.cbs.service.util.TemporalAmountParser).parse('${eu.covid.test.certificate.signing.cwt.recovery.minimum-period:P11D}')}")
    private TemporalAmount recoveryMinimumPeriod;

    @Value("#{T(dk.sundhedsdatastyrelsen.cbs.service.util.TemporalAmountParser).parse('${eu.covid.test.certificate.signing.cwt.recovery.maximum-period:P180D}')}")
    private TemporalAmount recoveryMaximumPeriod;

    @Value("#{T(dk.sundhedsdatastyrelsen.cbs.service.util.TemporalAmountParser).parse('${eu.covid.test.certificate.signing.cwt.ag-result.maximum-age:P7D}')}")
    private TemporalAmount antiGenResultsMaxAge;

    @Value("#{T(dk.sundhedsdatastyrelsen.cbs.service.util.TemporalAmountParser).parse('${eu.covid.test.certificate.signing.cwt.pcr-result.maximum-age:P7D}')}")
    private TemporalAmount pcrResultsMaxAge;
    @Value("#{T(dk.sundhedsdatastyrelsen.cbs.service.util.TemporalAmountParser).parse('${eu.covid.test.certificate.signing.allowed-clock-skew:PT5S}')}")
    private TemporalAmount allowedClockSkew;

    public TemporalAmount getCwtTestResultMaxValidity() {
        return cwtTestResultMaxValidity;
    }

    public TemporalAmount getCwtRecoveryMaxValidity() {
        return cwtRecoveryMaxValidity;
    }

    public TemporalAmount getRecoveryMinimumPeriod() {
        return recoveryMinimumPeriod;
    }

    public TemporalAmount getRecoveryMaximumPeriod() {
        return recoveryMaximumPeriod;
    }

    public TemporalAmount getAntiGenResultsMaxAge() {
        return antiGenResultsMaxAge;
    }

    public TemporalAmount getPcrResultsMaxAge() {
        return pcrResultsMaxAge;
    }

    public TemporalAmount getAllowedClockSkew() {
        return allowedClockSkew;
    }
}
