/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.config;

import dk.sundhedsdatastyrelsen.cbs.rules.EuRecoveryPassportEvaluator;
import dk.sundhedsdatastyrelsen.cbs.service.passports.model.EuRules;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.lang.NonNull;

import java.time.temporal.TemporalAmount;

@Configuration
@PropertySource("classpath:application-service.properties")
public class EuPassportConfiguration {
    @Value("#{T(dk.sundhedsdatastyrelsen.cbs.service.util.TemporalAmountParser).parse('${cwt.eu.maximum-validity:PT72H}')}")
    private TemporalAmount cwtEuMaximumValidity;
    @Value("#{T(dk.sundhedsdatastyrelsen.cbs.service.util.TemporalAmountParser).parse('${cwt.eu.ag-result.maximum-age:P7D}')}")
    private TemporalAmount antiGenResultsMaxAge;
    @Value("#{T(dk.sundhedsdatastyrelsen.cbs.service.util.TemporalAmountParser).parse('${cwt.eu.pcr-result.maximum-age:P7D}')}")
    private TemporalAmount pcrResultsMaxAge;
    @Value("${cwt.eu.include-positive-results:false}")
    private boolean includePositiveResults;

    @Value("${cwt.eu.ag-result.limit:50}")
    private int antiGenResultLimit;
    @Value("${cwt.eu.pcr-result.limit:50}")
    private int pcrResultLimit;

    public TemporalAmount getCwtEuMaximumValidity() {
        return cwtEuMaximumValidity;
    }

    public TemporalAmount getAntiGenResultsMaxAge() {
        return antiGenResultsMaxAge;
    }

    public TemporalAmount getPcrResultsMaxAge() {
        return pcrResultsMaxAge;
    }

    public boolean isIncludePositiveResults() {
        return includePositiveResults;
    }

    public int getAntiGenResultLimit() {
        return antiGenResultLimit;
    }

    public int getPcrResultLimit() {
        return pcrResultLimit;
    }


    @Value("#{T(dk.sundhedsdatastyrelsen.cbs.service.util.TemporalAmountParser).parse('${eu.recovery.validator.minimum-period-after-recovery:P14D}')}")
    private TemporalAmount minimumPeriodAfterRecovery;
    @Value("#{T(dk.sundhedsdatastyrelsen.cbs.service.util.TemporalAmountParser).parse('${eu.recovery.validator.maximum-period-after-recovery:P8M}')}")
    private TemporalAmount maximumPeriodAfterRecovery;

    @Bean
    @EuRules
    @NonNull
    public EuRecoveryPassportEvaluator euRecoveryPassportEvaluator() {
        return new EuRecoveryPassportEvaluator(minimumPeriodAfterRecovery, maximumPeriodAfterRecovery);
    }

    @Override
    public String toString() {
        return "EuPassportConfiguration{" +
            "cwtEuMaximumValidity=" + cwtEuMaximumValidity +
            ", antiGenResultsMaxAge=" + antiGenResultsMaxAge +
            ", pcrResultsMaxAge=" + pcrResultsMaxAge +
            ", includePositiveResults=" + includePositiveResults +
            ", antiGenResultLimit=" + antiGenResultLimit +
            ", pcrResultLimit=" + pcrResultLimit +
            ", minimumPeriodAfterRecovery=" + minimumPeriodAfterRecovery +
            ", maximumPeriodAfterRecovery=" + maximumPeriodAfterRecovery +
            '}';
    }
}
