/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.config;

import dk.sundhedsdatastyrelsen.cbs.service.passports.VaccineMapper;
import dk.sundhedsdatastyrelsen.cbs.service.passports.VaccineMappingConfiguration;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.Resource;
import org.yaml.snakeyaml.Yaml;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

@Configuration
public class VaccineMapperConfiguration {

    @Bean
    public VaccineMapper vaccineMapper(@Value("${vaccinemapping.path:vaccinemapping.yml}") Resource mapping) throws IOException {

        try (InputStream inputStream = mapping.getInputStream()) {
            HashMap<Long, HashMap<String, String>> raw = new Yaml().load(inputStream);
            Map<Long, VaccineMappingConfiguration.VaccineInformation> mappingTable = raw.entrySet().stream().collect(Collectors.toMap(Map.Entry::getKey, f -> new VaccineMappingConfiguration.VaccineInformation(f.getValue().get("vp"), f.getValue().get("mp"), f.getValue().get("ma"))));
            return new VaccineMapper(new VaccineMappingConfiguration(mappingTable));
        }
    }
}
