/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.mapper;

import dk.sundhedsdatastyrelsen.cbs.service.model.CoronaPass;
import dk.sundhedsdatastyrelsen.cbs.service.model.CoronaPassResult;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.FutureResult;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.GetCoronaPassResponse;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.KeyValuePair;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.ObjectFactory;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.PassJWT;

import java.util.Collection;
import java.util.Map;

public final class CoronaPassResponseMapper {

    private static final ObjectFactory OBJECT_FACTORY = new ObjectFactory();

    private CoronaPassResponseMapper() {
    }

    public static GetCoronaPassResponse toCoronaPassResponse(CoronaPassResult coronaPassResult) {
        if (coronaPassResult.isCacheHit()) {
            return CoronaPassResponseMapper.createCacheHitResponse(coronaPassResult.getCoronaPasses());
        } else {
            return CoronaPassResponseMapper.createCacheMissResponse(coronaPassResult.getJobId());
        }
    }

    private static GetCoronaPassResponse createCacheHitResponse(Collection<CoronaPass> coronaPasses) {
        if (coronaPasses == null) {
            throw new IllegalStateException("coronaPasses cannot be null when createCacheHitResponse is called");
        }

        GetCoronaPassResponse response = OBJECT_FACTORY.createGetCoronaPassResponse();
        if (!coronaPasses.isEmpty()) {
            response.setCoronaPasses(OBJECT_FACTORY.createPassList());
        }
        for (CoronaPass coronaPass : coronaPasses) {
            PassJWT passJWT = OBJECT_FACTORY.createPassJWT();
            passJWT.setType(coronaPass.getPassType().toString());
            passJWT.setJWT(coronaPass.getToken());
            for (Map.Entry<String, String> prop : coronaPass.getProperties().entrySet()) {
                KeyValuePair keyValuePair = OBJECT_FACTORY.createKeyValuePair();
                keyValuePair.setKey(prop.getKey());
                keyValuePair.setValue(prop.getValue());
                passJWT.getProperties().add(keyValuePair);
            }
            response.getCoronaPasses().getPass().add(passJWT);
        }
        return response;
    }

    private static GetCoronaPassResponse createCacheMissResponse(String jobId) {
        if (jobId == null) {
            throw new IllegalStateException("jobId cannot be null when createCacheMissResponse is called");
        }

        GetCoronaPassResponse response = OBJECT_FACTORY.createGetCoronaPassResponse();
        FutureResult futureResult = OBJECT_FACTORY.createFutureResult();
        futureResult.setJobId(jobId);
        response.setFutureResult(futureResult);
        return response;
    }


}
