/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.model;

import org.springframework.lang.NonNull;

import java.util.HashMap;
import java.util.Map;

public class CoronaPass {
    private final PassType passType;
    private final String token;
    private final Map<String, String> properties;

    public CoronaPass(@NonNull PassType passType, @NonNull String token, @NonNull Map<String, String> properties) {
        this.passType = passType;
        this.token = token;
        this.properties = properties;
    }

    public CoronaPass(@NonNull PassType passType, @NonNull String token) {
        this.passType = passType;
        this.token = token;
        this.properties = new HashMap<>();
    }

    @NonNull
    public PassType getPassType() {
        return passType;
    }

    @NonNull
    public String getToken() {
        return token;
    }

    @NonNull
    public Map<String, String> getProperties() {
        return properties;
    }

    /**
     * @return null or previous value if property was already set
     */
    public String setProperty(@NonNull String key, @NonNull String value) {
        return properties.put(key, value);
    }

    public boolean hasProperty(@NonNull String key, @NonNull String value) {
        return value.equals(properties.get(key));
    }
}
