/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.passports;

import dk.sundhedsdatastyrelsen.cbs.service.exceptions.VaccineMapperException;

import java.util.Map;

public class VaccineMappingConfiguration {

    private final Map<Long, VaccineInformation> mappingTable;

    public VaccineMappingConfiguration(Map<Long, VaccineInformation> mappingTable) {
        this.mappingTable = mappingTable;
    }

    public VaccineInformation getVaccineIdValues(Long vaccineIdentifier) {
        if (mappingTable.containsKey(vaccineIdentifier)) {
            return mappingTable.get(vaccineIdentifier);
        }

        throw new VaccineMapperException(String.format("Unknown drugIdentifier: '%s' - vaccine cannot be mapped", vaccineIdentifier));
    }

    public static class VaccineInformation {
        public final String vp;
        public final String mp;
        public final String ma;

        public VaccineInformation(String vp, String mp, String ma) {
            this.vp = vp;
            this.mp = mp;
            this.ma = ma;
        }
    }
}
