/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.webservice;

import dk.sds.nsp.security.SecurityContext;
import dk.sundhedsdatastyrelsen.cbs.service.CoronaPassService;
import dk.sundhedsdatastyrelsen.cbs.service.ErrorMessages;
import dk.sundhedsdatastyrelsen.cbs.service.exceptions.MissingInformationException;
import dk.sundhedsdatastyrelsen.cbs.service.mapper.CoronaPassResponseMapper;
import dk.sundhedsdatastyrelsen.cbs.service.model.CoronaPassResult;
import dk.sundhedsdatastyrelsen.cbs.service.security.WsSecurity;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.CoronaPassPort;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.GetCoronaPassRequest;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.GetCoronaPassResponse;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.ObjectFactory;
import org.apache.cxf.annotations.SchemaValidation;
import org.springframework.context.ApplicationContext;
import org.springframework.lang.NonNull;

import javax.jws.HandlerChain;
import javax.jws.WebMethod;
import javax.jws.WebResult;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding;
import javax.xml.bind.annotation.XmlSeeAlso;

@SchemaValidation(type = SchemaValidation.SchemaValidationType.BOTH)
@WebService(name = "CoronaPassPort", targetNamespace = "http://sundhedsdatastyrelsen.dk/coronapass/2021/03/coronapass-web-service")
@SOAPBinding(parameterStyle = SOAPBinding.ParameterStyle.BARE)
@HandlerChain(file = "handler-chain.xml")
@XmlSeeAlso({
    ObjectFactory.class
})
public class CoronaPassWebService extends AbstractWebService implements CoronaPassPort {

    public static final String SERVICE_IDENTIFIER = "CoronaPassWebService";

    private CoronaPassService coronaPassService;
    private WsSecurity wsSecurity;

    @Override
    protected void initializeSpringDependencies(@NonNull final ApplicationContext context) {
        coronaPassService = context.getBean(CoronaPassService.class);
        wsSecurity = context.getBean(WsSecurity.class);
    }

    @Override
    @WebMethod(operationName = "GetCoronaPass")
    @WebResult(name = "GetCoronaPassResponse", targetNamespace = "http://sundhedsdatastyrelsen.dk/coronapass/2021/03/coronapass-web-service", partName = "GetCoronaPassResponse")
    public GetCoronaPassResponse getCoronaPass(GetCoronaPassRequest request) {
        SecurityContext securityContext = dk.sds.nsp.security.Security.getSecurityContext();
        wsSecurity.checkSecurityTicket(securityContext, request.getIdentifier(), SERVICE_IDENTIFIER);
        final String idwsMessageId = securityContext.getMessage().flatMap(SecurityContext.Message::getIdentifier).orElseThrow(() -> new MissingInformationException(ErrorMessages.MISSING_MESSAGE_ID));
        CoronaPassResult coronaPassResult = coronaPassService.getCoronaPass(request.getIdentifier(), idwsMessageId);
        return CoronaPassResponseMapper.toCoronaPassResponse(coronaPassResult);
    }
}
