/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.webservice;

import dk.sds.nsp.security.SecurityContext;
import dk.sundhedsdatastyrelsen.cbs.service.ErrorMessages;
import dk.sundhedsdatastyrelsen.cbs.service.exceptions.MissingInformationException;
import org.apache.log4j.MDC;

import javax.xml.namespace.QName;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;
import java.util.Collections;
import java.util.Set;
import java.util.UUID;

import static dk.sundhedsdatastyrelsen.cbs.CbsLogging.*;

public class LoggingSoapHandler implements SOAPHandler<SOAPMessageContext> {
    @Override
    public Set<QName> getHeaders() {
        return Collections.emptySet();
    }

    @Override
    public boolean handleMessage(SOAPMessageContext context) {
        boolean isInbound = Boolean.FALSE.equals(context.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY));
        if (isInbound) {
            MDC.put(TRACE_ID_PROP, UUID.randomUUID().toString());
            SecurityContext securityContext = dk.sds.nsp.security.Security.getSecurityContext();
            final String idwsMessageId = securityContext.getMessage().flatMap(SecurityContext.Message::getIdentifier).orElseThrow(()-> new MissingInformationException(ErrorMessages.MISSING_MESSAGE_ID));
            MDC.put(MESSAGE_ID_PROP, idwsMessageId);
        }
        return true;
    }

    @Override
    public boolean handleFault(SOAPMessageContext context) {
        return true;
    }

    @Override
    public void close(MessageContext context) {
        MDC.remove(TRACE_ID_PROP);
        MDC.remove(MESSAGE_ID_PROP);
        MDC.remove(JOB_ID_PROP);
    }
}
