/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.config;

import com.google.common.collect.ImmutableMap;
import dk.sds.nsp.kafka.clients.producer.NSPKafkaProducer;
import dk.sundhedsdatastyrelsen.cbs.dto.kafka.DataFetcherJob;
import dk.sundhedsdatastyrelsen.cbs.dto.kafka.FetchType;
import dk.sundhedsdatastyrelsen.cbs.kafka.CbsKafkaProducer;
import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.Mock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.test.context.TestPropertySource;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;

import java.util.Properties;
import java.util.UUID;
import java.util.function.Function;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.*;

@CbsTest
@SpringJUnitConfig({ServiceKafkaConfig.class, CoronaPassServiceKafkaConfigTest.MockConfig.class})
@TestPropertySource(properties = {
    "kafka.producer.bootstrap.servers=somehost:9999",
    "kafka.producer.client.id=unittest",
    "kafka.topic=my-topic"
})
class CoronaPassServiceKafkaConfigTest {
    @Autowired
    private Function<Properties, NSPKafkaProducer<String, DataFetcherJob>> mockNspKafkaProducerConstructor;
    @Captor
    ArgumentCaptor<Properties> propertiesArgumentCaptor;

    @Configuration
    public static class MockConfig {
        @Bean
        @Primary
        public Function<Properties, NSPKafkaProducer<String, DataFetcherJob>> nspKafkaProducerConstructor() {
            return mock(Function.class);
        }
    }

    @Mock
    private NSPKafkaProducer<String, DataFetcherJob> producerMock;

    @Autowired
    private CbsKafkaProducer cbsKafkaProducer;

    @Test
    void makes_kafka_producer() {
        assertNotNull(cbsKafkaProducer);
        when(mockNspKafkaProducerConstructor.apply(any())).thenReturn(producerMock);

        final String cpr = "0506504003";
        cbsKafkaProducer.sendDataFetcherJob(cpr, UUID.randomUUID().toString(), UUID.randomUUID().toString(), ImmutableMap.of(cpr, FetchType.FETCH_ALL));

        verify(mockNspKafkaProducerConstructor).apply(propertiesArgumentCaptor.capture());
        assertEquals("unittest", propertiesArgumentCaptor.getValue().getProperty(ProducerConfig.CLIENT_ID_CONFIG));

    }
}
