/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.config;


import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import org.apache.log4j.Appender;
import org.apache.log4j.Logger;
import org.apache.log4j.spi.LoggingEvent;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.Mock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.TestPropertySource;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;

import static com.google.common.truth.Truth.assertThat;
import static org.mockito.Mockito.verify;

@CbsTest
@SpringJUnitConfig({DkPassportConfiguration.class, EuPassportConfiguration.class, SignatureConfiguration.class, VaccineMapperConfiguration.class})
@ActiveProfiles(SignatureConfiguration.LOCAL_KEY_STORE)
@TestPropertySource(properties = {
    "cwt.children.include-in-response=true",
    "cwt.children.include-in-response.dk-passes=true",
    "cwt.eu.maximum-validity=P22M",
    "cwt.dk.maximum-validity=P42M"
})
class ServiceConfigTest {
    private final ServiceConfig serviceConfig = new ServiceConfig();

    @Mock
    private Appender appender;
    @Captor
    private ArgumentCaptor<LoggingEvent> loggingEventArgumentCaptor;
    @Autowired
    private DkPassportConfiguration dkPassportConfiguration;
    @Autowired
    private EuPassportConfiguration euPassportConfiguration;

    @BeforeEach
    void setUp() {
        Logger logger = Logger.getLogger("SYSTEM_AUDIT");
        logger.addAppender(appender);
    }

    @Test
    void logs_passport_configuration() {
        assertThat(dkPassportConfiguration.toString()).isNotEqualTo(euPassportConfiguration.toString());

        serviceConfig.auditLogger(dkPassportConfiguration, euPassportConfiguration, true);

        verify(appender).doAppend(loggingEventArgumentCaptor.capture());
        LoggingEvent loggingEvent = loggingEventArgumentCaptor.getValue();

        assertThat(loggingEvent.getRenderedMessage()).contains(dkPassportConfiguration.toString());
        assertThat(loggingEvent.getRenderedMessage()).contains("cwt.children.include-in-response=\"true\"");
        assertThat(loggingEvent.getRenderedMessage()).contains("makeChildrenDkPasses=true");
        assertThat(loggingEvent.getRenderedMessage()).contains("cwtEuMaximumValidity=P22M");
        assertThat(loggingEvent.getRenderedMessage()).contains("cwtDkMaximumValidity=P42M");
        assertThat(loggingEvent.getRenderedMessage()).contains(euPassportConfiguration.toString());
    }
}
