/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.passports;


import org.junit.jupiter.api.Test;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;

class ChecksumUtilsTest {

    private static final String NAMESPACE_URL = "6ba7b811-9dad-11d1-80b4-00c04fd430c8";

    @Test
    void version_5_UUID_is_correctly_generated_for_domain_baeldung_com() {

        UUID uuid = ChecksumUtils.generateType5UUID(NAMESPACE_URL, "laegemiddelstyrelsen.dk");

        assertEquals("dd7ed381-f31f-5b46-ba17-db7ca51a5c2b", uuid.toString());
        assertEquals(5, uuid.version());
        assertEquals(2, uuid.variant());
    }

    @Test
    void validateLuhnModNChecksum() {
        assertEquals('Z', ChecksumUtils.generateLuhnModNCheckCharacter("URN:UVCI:01:NL:187/37512422923"));
        assertEquals('E', ChecksumUtils.generateLuhnModNCheckCharacter("01LUX/18737512422923"));
        assertEquals('N', ChecksumUtils.generateLuhnModNCheckCharacter("01:SE:EHM/123456789KOOZC6RM7VZ"));
    }

    @Test
    void testUVCI() throws URISyntaxException {
        String result = ChecksumUtils.generateUVCI("2");
        assertEquals("URN:UVCI:01:DK:B19D10B4E18551559EBDEE46248DA883#S", result);
        assertEquals("S", new URI(result).getFragment());

    }
}
