/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.passports;

import dk.sundhedsdatastyrelsen.cbs.dto.impl.vaccination.VaccinationBuilder;
import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResults;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.Vaccination;
import dk.sundhedsdatastyrelsen.cbs.logging.AuditData;
import dk.sundhedsdatastyrelsen.cbs.service.model.CoronaPass;
import dk.sundhedsdatastyrelsen.cbs.service.model.PassType;
import org.junit.jupiter.api.BeforeAll;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.datasource.DriverManagerDataSource;
import org.springframework.jdbc.datasource.lookup.JndiDataSourceLookup;
import org.springframework.mock.jndi.SimpleNamingContextBuilder;

import javax.naming.NamingException;
import java.util.Collection;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static dk.sundhedsdatastyrelsen.cbs.service.passports.TestData.PERSON_RECORD;
import static java.time.LocalDateTime.now;
import static org.mockito.Mockito.mock;

public class EUAntigenTest {
    final Vaccination noVaccinations = VaccinationBuilder.vaccination(TestData.CPR, false).build();
    @Autowired
    protected CombinedPassportFactory combinedPassportFactory;

    @BeforeAll
    static void beforeAll() throws NamingException {
        //noinspection deprecation
        SimpleNamingContextBuilder.emptyActivatedContextBuilder();
        new JndiDataSourceLookup().getJndiTemplate().bind("java:jboss/datasources/euantigenspec", new DriverManagerDataSource("jdbc:h2:mem:testdb"));
    }

    protected Collection<CoronaPass> getEUPass(Collection<CoronaPass> pass, String euType) {
        return getPassByType(pass, PassType.EU)
            .filter(coronaPass -> coronaPass.hasProperty("eu:keytype", euType))
            .collect(Collectors.toList());
    }

    protected Stream<CoronaPass> getPassByType(Collection<CoronaPass> pass, PassType type) {
        return pass.stream().filter(coronaPass -> coronaPass.getPassType() == type);
    }

    protected Collection<CoronaPass> getCoronaPasses(Vaccination vaccination, LabResults labResults) {
        return combinedPassportFactory.createPassportFrom(vaccination, labResults, PERSON_RECORD, new AuditData(), null);
    }

}
