-- liquibase formatted sql
-- changeset cdj@trifork.com:test-vaccination

create or replace temporary sequence seq_trace_id;

create or replace view vaccination_debug as
select if(vv.cpr is null, null, '*') active, if(v.data is null, 'invalidation', 'write') type, v.*
from vaccination v
         left join vaccination_view vv on v.cpr = vv.cpr and v.creation_time = vv.creation_time
order by cpr, creation_time;

create or replace view lab_result_debug as
select if(vv.cpr is null, null, '*') active, if(v.data is null, 'invalidation', 'write') type, v.*
from lab_result v
         left join lab_result_view vv on v.cpr = vv.cpr and v.creation_time = vv.creation_time
order by cpr, creation_time;

create or replace view debug as
select * from vaccination_debug
union
select * from lab_result_debug
order by cpr, creation_time, data;
-- Eternal write
set @cpr = random_cpr();
set @data = random_string();

insert into vaccination (cpr, trace_id, creation_time, invalidation_time, data)
values (@cpr, nextval(seq_trace_id), current_timestamp, null, @data);
call assert_equals(@data, (select data from vaccination_view where cpr = @cpr));

-- Unexpired write
set @cpr = random_cpr();
set @data = random_string();
insert into vaccination (cpr, trace_id, creation_time, invalidation_time, data)
values (@cpr, nextval(seq_trace_id), date_sub(current_timestamp, interval 2 hour), date_add(current_timestamp, interval 1 hour), @data);

call assert_equals(@data, (select data from vaccination_view where cpr = @cpr));

-- Expired write
set @cpr = random_cpr();
set @data = random_string();
insert into vaccination (cpr, trace_id, creation_time, invalidation_time, data)
values (@cpr, nextval(seq_trace_id), date_sub(current_timestamp, interval 2 hour), date_sub(current_timestamp, interval 1 hour), @data);

call assert_null((select data from vaccination_view where cpr = @cpr));

-- Eternal write and invalidation recorded after write
set @cpr = random_cpr();
set @data = random_string();
insert into vaccination (cpr, trace_id, creation_time, invalidation_time, data)
values (@cpr, nextval(seq_trace_id), date_sub(current_timestamp, interval 2 hour), null, @data);
insert into vaccination (cpr, trace_id, creation_time, invalidation_time, data)
values (@cpr, nextval(seq_trace_id), date_sub(current_timestamp, interval 1 hour), date_sub(current_timestamp, interval 1 microsecond), null);

call assert_null((select data from vaccination_view where cpr = @cpr));

-- Eternal write and invalidation recorded before write
set @cpr = random_cpr();
set @data = random_string();
insert into vaccination (cpr, trace_id, creation_time, invalidation_time, data)
values (@cpr, nextval(seq_trace_id), date_sub(current_timestamp, interval 2 hour), null, @data);
insert into vaccination (cpr, trace_id, creation_time, invalidation_time, data)
values (@cpr, nextval(seq_trace_id), date_sub(current_timestamp, interval 3 hour), date_sub(current_timestamp, interval 1 hour), null);

call assert_null((select data from vaccination_view where cpr = @cpr));

-- Expired invalidation and eternal write after expiration
set @cpr = random_cpr();
set @data = random_string();
insert into vaccination (cpr, trace_id, creation_time, invalidation_time, data)
values (@cpr, nextval(seq_trace_id), date_sub(current_timestamp, interval 1 hour), null, @data);
insert into vaccination (cpr, trace_id, creation_time, invalidation_time, data)
values (@cpr, nextval(seq_trace_id), date_sub(current_timestamp, interval 3 hour), date_sub(current_timestamp, interval 2 hour ), null);

call assert_equals(@data, (select data from vaccination_view where cpr = @cpr));

-- Eternal write overwritten by new eternal write
set @cpr = random_cpr();
set @data = random_string();
insert into vaccination (cpr, trace_id, creation_time, invalidation_time, data)
values (@cpr, nextval(seq_trace_id), date_sub(current_timestamp, interval 2 hour), null, @data);
set @data = random_string();
insert into vaccination (cpr, trace_id, creation_time, invalidation_time, data)
values (@cpr, nextval(seq_trace_id), date_sub(current_timestamp, interval 1 hour), null, @data);

call assert_equals(@data, (select data from vaccination_view where cpr = @cpr));

-- Eternal write and expired invalidation after write, e.g. negative invalidation offset
set @cpr = random_cpr();
set @data = random_string();
insert into vaccination (cpr, trace_id, creation_time, invalidation_time, data)
values (@cpr, nextval(seq_trace_id), date_sub(current_timestamp, interval 2 hour), null, @data);
insert into vaccination (cpr, trace_id, creation_time, invalidation_time, data)
values (@cpr, nextval(seq_trace_id), date_sub(current_timestamp, interval 1 hour), date_sub(current_timestamp, interval 3 hour), null);

call assert_equals(@data, (select data from vaccination_view where cpr = @cpr));


truncate table vaccination;
