/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package se.digg.dgc.cose;

import org.junit.Assert;
import org.junit.Test;

import com.upokecenter.cbor.CBORObject;

import se.digg.dgc.signatures.cose.SignatureAlgorithm;

/**
 * Test cases for SignatureAlgorithmId.
 * 
 * @author Martin Lindström (martin@idsec.se)
 * @author Henrik Bengtsson (extern.henrik.bengtsson@digg.se)
 * @author Henric Norlander (extern.henric.norlander@digg.se)
 */
public class SignatureAlgorithmTest {

  @Test
  public void testFromValue() throws Exception {
    SignatureAlgorithm id = SignatureAlgorithm.fromValue(-7);
    Assert.assertEquals(SignatureAlgorithm.ES256, id);
    
    id = SignatureAlgorithm.fromValue(-35);
    Assert.assertEquals(SignatureAlgorithm.ES384, id);
    
    id = SignatureAlgorithm.fromValue(-36);
    Assert.assertEquals(SignatureAlgorithm.ES512, id);
    
    id = SignatureAlgorithm.fromValue(-37);
    Assert.assertEquals(SignatureAlgorithm.PS256, id);
    
    id = SignatureAlgorithm.fromValue(-38);
    Assert.assertEquals(SignatureAlgorithm.PS384, id);
    
    id = SignatureAlgorithm.fromValue(-39);
    Assert.assertEquals(SignatureAlgorithm.PS512, id);
  }
  
  @Test(expected = IllegalArgumentException.class)
  public void testFromValueNoMatch() throws Exception {
    SignatureAlgorithm.fromValue(47);
  }
  
  @Test
  public void testFromCborObject() throws Exception {
    SignatureAlgorithm id = SignatureAlgorithm.fromCborObject(CBORObject.FromObject(-7));
    Assert.assertEquals(SignatureAlgorithm.ES256, id);
    
    id = SignatureAlgorithm.fromCborObject(CBORObject.FromObject(-35));
    Assert.assertEquals(SignatureAlgorithm.ES384, id);
    
    id = SignatureAlgorithm.fromCborObject(CBORObject.FromObject(-36));
    Assert.assertEquals(SignatureAlgorithm.ES512, id);
    
    id = SignatureAlgorithm.fromCborObject(CBORObject.FromObject(-37));
    Assert.assertEquals(SignatureAlgorithm.PS256, id);
    
    id = SignatureAlgorithm.fromCborObject(CBORObject.FromObject(-38));
    Assert.assertEquals(SignatureAlgorithm.PS384, id);
    
    id = SignatureAlgorithm.fromCborObject(CBORObject.FromObject(-39));
    Assert.assertEquals(SignatureAlgorithm.PS512, id);
  }
  
  @Test(expected = IllegalArgumentException.class)
  public void testFromCborObjectNoMatch() throws Exception {
    SignatureAlgorithm.fromCborObject(CBORObject.False);
  }
  
  @Test
  public void testGetValue() throws Exception {
    SignatureAlgorithm id = SignatureAlgorithm.fromValue(-7);
    Assert.assertEquals(-7, id.getValue());
  }
  
  @Test
  public void testGetCborObject() throws Exception {
    SignatureAlgorithm id = SignatureAlgorithm.fromValue(-7);
    Assert.assertEquals(CBORObject.FromObject(-7), id.getCborObject());
  }
  
}
