/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package se.digg.dgc.cwt;

import java.time.Duration;
import java.time.Instant;

import org.junit.Assert;
import org.junit.Test;

import com.upokecenter.cbor.CBORObject;

import se.digg.dgc.signatures.cwt.Cwt;

/**
 * Test cases for the Cwt class.
 *
 * @author Martin Lindström (martin@idsec.se)
 * @author Henrik Bengtsson (extern.henrik.bengtsson@digg.se)
 * @author Henric Norlander (extern.henric.norlander@digg.se)
 */
public class CwtTest {

  @Test
  public void testCreate() {
    
    final Instant now = Instant.now(); 
    final long seconds = now.getEpochSecond();
    
    final CBORObject object = CBORObject.FromObject("value");
    
    Cwt cwt = Cwt.builder()
      .issuer("Kalle")
      .issuedAt(now)
      .expiration(now.plus(Duration.ofDays(30)))
      .claim("98", object.EncodeToBytes())
      .claim(99, object)
      .build();
    
    Cwt cwt2 = Cwt.decode(cwt.encode());
    
    Assert.assertEquals("Kalle", cwt2.getIssuer());
    Assert.assertEquals(seconds, cwt2.getIssuedAt().getEpochSecond());
    Assert.assertEquals("value", cwt2.getClaim("98").AsString());
    Assert.assertEquals("value", cwt2.getClaim(99).AsString());
  }
  
}
