/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package se.digg.dgc.service.impl;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import se.digg.dgc.payload.v1.DGCSchemaException;
import se.digg.dgc.payload.v1.DigitalGreenCertificate;
import se.digg.dgc.signatures.CertificateProvider;
import se.digg.dgc.signatures.impl.DefaultDGCSignatureVerifier;

import java.io.IOException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.SignatureException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateExpiredException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;

public class DefaultDGCDecoderTest {

    private static Certificate certificate;
    private static DefaultDGCSignatureVerifier verifier;
    private static CertificateProvider certificateProvider;

    @BeforeAll
    public static void beforeAll() throws KeyStoreException, CertificateException, IOException, NoSuchAlgorithmException {
        KeyStore keystore = KeyStore.getInstance("PKCS12");
        keystore.load(DefaultDGCDecoderTest.class.getClassLoader().getResourceAsStream("Sundhedsdatastyrelsen - COVID-19 Borgerstatus.p12"), "Test1234".toCharArray());
        certificate = keystore.getCertificate("covid-19 borgerstatus (funktionscertifikat)");
        verifier = new DefaultDGCSignatureVerifier();
        certificateProvider = (country, kid) -> {
            List<X509Certificate> listOfCertificate = new ArrayList<>();
            listOfCertificate.add((X509Certificate) certificate);
            return listOfCertificate;
        };
    }

    public static DigitalGreenCertificate getEUPayload(String token) throws CertificateExpiredException, DGCSchemaException, SignatureException, IOException {
        return new DefaultDGCDecoder(verifier, certificateProvider).decode(token);
    }

    @Test
    @Disabled("Used for manual inspection")
    void parseEUToken() throws Exception {
        String pass = "HC1:NCFX600G0A2WF.9.60+O4W79F%GZM9NTVO KFBB853*70NS8FN0R9CZ8PWY0N9C9TQD97TK0H90GECFAGYWEKECTHGWJC0FDRX5:KEPH7.$E8UCWG6V474R63S65W5HOCY964S6 47CW5Y C346CL6-96IPCTZC1R6Z CWJCT3ENS8XJC$+DXJCCWENF6OF63W5$Q6TF6%JC+QE P3CEC+EDG7D%69AECAWEP34IECOCCHC8AWE04E-EDOFFB$D% D3IA4W5646946846.96.JCP9EJY8L/5M/5546.96VF63KC.SC4KCD3DX47B46IL6646H*6Z/E5JD%96IA74R6646407GVC*JC1A61:63W5Q47AL6TPCBEC7ZKW.C7 C-ED7WE JC5/DK19* C5$CLQE:.DW.C7WEO09XTAXY8I3DBWEX095Y8GY8WNA.+9O/EZKEZ96446256V50WP9B TD1LIC3+98X+9PL5Z-US8E4Q7XM3 WQYBGBF261R706WTRKFT5B9ZHE 19OCT.S90RTWMH9UB73PQ0KJK2 OTO-MG7KK+MN2UKS7AFKE64QQBW19B-633O 7Q++D M9TI3E7GC27PO4RF25PENH7WP4JVF6EL F4OF161W0VSWCB:P2G2D6SE+ZEXUCV.BH3WZ:M-27 +IXW1J OEBH0T6:R6-9OC9CTGK9:P*WPEUG8MIVZ90%RFUANMN YAJ*TD+FAANSJE49SF6EONIK93-VH+3DCQ3R0BFGJ676.K3UCS%PR8NB:RP%7OR2M$:V4LGMCDCZ3P5TETQ8G3H*4J42SJR*3SV.91MAO%MJTBC$6 KKNVUILL0MC680IOEF%R";
        DigitalGreenCertificate cert = getEUPayload(pass.substring(4));
        System.out.println(cert.toJSONString());
    }

}
