/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package se.digg.dgc.service.impl;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import se.digg.dgc.signatures.CertificateProvider;
import se.digg.dgc.signatures.cwt.Cwt;
import se.digg.dgc.signatures.impl.DefaultDGCSignatureVerifier;

import java.io.IOException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;

/**
 * Test cases for the {@link DefaultDGCDecoder} class.
 */
public class DefaultDKDecoderTest {

    private static Certificate certificate;
    private static DefaultDGCSignatureVerifier verifier;
    private static CertificateProvider certificateProvider;

    @BeforeAll
    public static void beforeAll() throws KeyStoreException, CertificateException, IOException, NoSuchAlgorithmException {
        KeyStore keystore = KeyStore.getInstance("PKCS12");
        keystore.load(DefaultDKDecoderTest.class.getClassLoader().getResourceAsStream("Sundhedsdatastyrelsen - COVID-19 Borgerstatus.p12"), "Test1234".toCharArray());
        certificate = keystore.getCertificate("covid-19 borgerstatus (funktionscertifikat)");
        verifier = new DefaultDGCSignatureVerifier();
        certificateProvider = (country, kid) -> {
            List<X509Certificate> listOfCertificate = new ArrayList<>();
            listOfCertificate.add((X509Certificate) certificate);
            return listOfCertificate;
        };
    }

    @Test
    @Disabled("Used for manual inspection")
    void parseEUToken() throws Exception {
        String pass = "DK2:NCFN80O80-7WF.9.60+O4W79F%GZM9NTVO K/7BH:K1JCYL92F3+ JMR92F3HEJVZPJ6BXZCYZC4:6%ZC6W5WIC9%63R61R67X52VC$Q6OB73UCSG6XJCM.CQF6XED* CAEC++9F$D23E-ED7WES34ZKE93D4LE1$C.PDIECF CX-C*EDAWE5JDMA76L6E466W5XA6V50PXC868WEA5KF7LSYWSX/RS19+QSZL1RX2VKTF26$ MP80SMRN:RIBSD6FW$3XENZ5O2GWSQ3AYQ976 LL3Z55%5NCJEPHZTF%GI.+G3COENMM03S G8L0*MCPANJK0.JM34C HS1U2MOV.GQ:0S4J2ST5D.LG%EX-CD%7Y*ARDRZ/STV4KWDFA12PTF I+XHIV5DI98MORDI*HBETPL 2UZ9%SBG0S3IFN6HZ.LF83 XDC353/2FY05G6:$RT:BUPDI7M:C7HQ7ZJF0TA182CS0M:NKXLT0U4B9J$2DT9TJ4%JTVOL9KV N6+0RF84LY8ZIJEP7CW2GUVGCC518$ 59B1 0C-B3+$58KCNXRSD54$U%O3L7J%I6H8CJRK/P3L:KPTK";
        Cwt cwt = new DefaultDKDecoder(certificate).getPayload(pass.substring(4));
        System.out.println(cwt.toString());
    }

}
