/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.fluent.spring.jdbc;

import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

public final class TableMetaDataBuilder {
    @Nullable private String catalogName;
    @Nullable private String schemaName;
    @NonNull private String tableName;

    private TableMetaDataBuilder(@NonNull final String tableName) {
        this.tableName = tableName;
    }

    @NonNull
    public static TableMetaDataBuilder tableMetaData(@NonNull final String tableName) {
        return new TableMetaDataBuilder(tableName);
    }

    @NonNull
    public TableMetaDataBuilder catalogName(@Nullable final String catalogName) {
        this.catalogName = catalogName;
        return this;
    }

    @NonNull
    public TableMetaDataBuilder schemaName(@Nullable final String schemaName) {
        this.schemaName = schemaName;
        return this;
    }

    @NonNull
    public TableMetaDataBuilder tableName(@NonNull final String tableName) {
        this.tableName = tableName;
        return this;
    }

    @NonNull
    public TableMetaData build() {
        return new TableMetaDataImpl(catalogName, schemaName, tableName);
    }
}
