/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.cbsdriver;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;
import java.util.Collections;
import java.util.Iterator;
import java.util.Set;

public class RequestModifyingSoapHandler implements SOAPHandler<SOAPMessageContext> {
    @Override
    public Set<QName> getHeaders() {
        return Collections.emptySet();
    }

    public static final Context CONTEXT = new Context();

    @Override
    public boolean handleMessage(SOAPMessageContext context) {
        boolean isOutBound = (Boolean) context.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);
        if (isOutBound && CONTEXT.shouldReplace()) {
            try {
                final SOAPMessage message = context.getMessage();
                final SOAPBody soapBody = message.getSOAPBody();

                replaceElement(soapBody.getChildNodes());

                message.saveChanges();
            } catch (SOAPException e) {
                throw new RuntimeException(e);
            }
        }
        return true;
    }

    private boolean replaceElement(NodeList nodes) {

        for (int i = 0; i < nodes.getLength(); i++) {
            final Node node = nodes.item(i);

            if (node.getLocalName() == null) {
                return false;
            } else if (node.getLocalName().equals(CONTEXT.getElementName())) {
                node.setTextContent(CONTEXT.getValue());
                return true;
            } else {
                final boolean found = replaceElement(node.getChildNodes());
                if (found) {
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public boolean handleFault(SOAPMessageContext context) {
        return true;
    }

    @Override
    public void close(MessageContext context) {
        // nothing to do
    }

    public static class Context {
        private String elementName;
        private String value;
        private boolean replace;

        public String getElementName() {
            return elementName;
        }

        public Context setElementName(String elementName) {
            this.elementName = elementName;
            return this;
        }

        public String getValue() {
            return value;
        }

        public Context setValue(String value) {
            this.value = value;
            return this;
        }

        public boolean shouldReplace() {
            return replace;
        }

        public Context setReplace(boolean replace) {
            this.replace = replace;
            return this;
        }
    }
}
