/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datacache;

import dk.sundhedsdatastyrelsen.cbs.webservice.idws.IdwsUser;
import org.springframework.jdbc.core.JdbcTemplate;

import java.time.LocalDateTime;
import java.util.UUID;

public class DataCacheDriver {
    private final JdbcTemplate jdbcTemplate;

    public DataCacheDriver(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    public Citizen withUser(IdwsUser user) {
        return new Citizen(user);
    }

    @SuppressWarnings("UnusedReturnValue")
    public class Citizen {

        private final IdwsUser user;

        public Citizen(IdwsUser user) {
            this.user = user;
        }

        public Citizen invalidateLabCache() {
            jdbcTemplate.update("insert into lab_result value (?,?,?,?,?)", user.cpr(), UUID.randomUUID().toString(), LocalDateTime.now(), LocalDateTime.now(), null);
            return this;
        }

        public Citizen invalidateVacCache() {
            jdbcTemplate.update("insert into vaccination value (?,?,?,?,?)", user.cpr(), UUID.randomUUID().toString(), LocalDateTime.now(), LocalDateTime.now(), null);
            return this;
        }


    }

}
