/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.ddvdriver.dgws;


import java.util.Collections;
import java.util.Set;

import javax.xml.bind.JAXBContext;
import javax.xml.namespace.QName;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;

import dk.sdsd.dgws._2010._08.NameFormat;
import dk.sdsd.dgws._2010._08.OrgUsingID;

public class WhitelistingClientMessageHandler implements SOAPHandler<SOAPMessageContext> {
    private final String systemOwnerName;
    private final String systemName;
    private final String clientVersion;
    private final String orgResponsibleName;
    private static final QName whitelistingHeaderQName = new QName("http://www.sdsd.dk/dgws/2012/06", "WhitelistingHeader");
    private  JAXBContext jaxbContext;

    public WhitelistingClientMessageHandler(String clientSystemOwnerName, String clientSystemName, String clientVersion, String orgResponsibleName) {
        systemOwnerName = clientSystemOwnerName;
        systemName = clientSystemName;
        this.clientVersion = clientVersion;
        this.orgResponsibleName = orgResponsibleName;

        try {
            jaxbContext =JAXBContext.newInstance(WhitelistingHeader.class);
        }
        catch(Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public Set<QName> getHeaders() {
        return Collections.singleton(whitelistingHeaderQName);
    }

    @Override
    public boolean handleMessage(SOAPMessageContext soapMessageContext) {
        String sysName = this.systemName;
        String version = this.clientVersion;

        WhitelistingHeader whitelistingHeader = new WhitelistingHeader();
        whitelistingHeader.setSystemOwnerName(systemOwnerName);
        whitelistingHeader.setSystemName(sysName);
        whitelistingHeader.setSystemVersion(version);
        whitelistingHeader.setRequestedRole("Læge");

        whitelistingHeader.setOrgResponsibleName(orgResponsibleName);

        OrgUsingID orgUsingId = new OrgUsingID();
        whitelistingHeader.setOrgUsingID(orgUsingId);

        orgUsingId.setValue("19");
        orgUsingId.setNameFormat(NameFormat.MEDCOM_YNUMBER);
        
        whitelistingHeader.setOrgUsingName("Ole Abrahamson");

        try {
            jaxbContext.createMarshaller().marshal(whitelistingHeader, soapMessageContext.getMessage().getSOAPHeader());
            return true;
        }
        catch(Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public boolean handleFault(SOAPMessageContext soapMessageContext) {
        return true;
    }

    @Override
    public void close(MessageContext messageContext) {
    }
}
