/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.kidsdriver;

import dk.sundhedsdatastyrelsen.cbs.test.RandomData;
import dk.sundhedsdatastyrelsen.cbs.webservice.idws.IdwsUser;
import org.springframework.jdbc.core.JdbcTemplate;

import java.time.LocalDateTime;

public class KidsDriver {
    private static final RandomData RANDOM_DATA = new RandomData(System.currentTimeMillis());

    private final JdbcTemplate jdbcTemplate;

    public KidsDriver(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    public Citizen withUser(IdwsUser user) {
        return new Citizen(user);
    }

    @SuppressWarnings({"unused", "UnusedReturnValue"})
    public class Citizen {

        private final IdwsUser user;

        public Citizen(IdwsUser user) {
            this.user = user;
        }

        public Citizen clearKids() {
            jdbcTemplate.update("delete from test_result where person_identifier = ?", user.cpr());
            return this;
        }

        public Citizen addPositivePcr(LocalDateTime samplingDateTime) {
            return addResult(true, "SARS2", samplingDateTime, "");
        }

        public Citizen addNegativePcr(LocalDateTime samplingDateTime) {
            return addResult(false, "SARS2", samplingDateTime, "");
        }

        public Citizen addPositiveAg(LocalDateTime samplingDateTime) {
            return addResult(true, "SARSG", samplingDateTime, "");
        }

        public Citizen addPositiveAg(LocalDateTime samplingDateTime, String manufacturer) {
            return addResult(true, "SARSG", samplingDateTime, manufacturer);
        }

        public Citizen addNegativeAg(LocalDateTime samplingDateTime) {
            return addResult(false, "SARSG", samplingDateTime, "");
        }

        public Citizen addNegativeAg(LocalDateTime samplingDateTime, String manufacturer) {
            return addResult(false, "SARSG", samplingDateTime, manufacturer);
        }

        public Citizen addResult(boolean positive, String type, LocalDateTime samplingDateTime, String manufacturer) {
            jdbcTemplate.update("insert into test_result " +
                "(test_identifier, person_identifier, type_of_test, test_name, test_manufacturer, result_of_the_test, testing_centre_or_facility, sletteflag, timestamp_sample, timestamp_result)" +
                "value (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", Math.abs(RANDOM_DATA.nextLong()), user.cpr(), type, "", manufacturer, positive ? "POSITIV" : "NEGATIV", "SEI kommunal- og skoleindberetninger", 0, samplingDateTime, samplingDateTime.plusHours(1));
            return this;
        }

    }

}
