/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.util;

import com.google.common.collect.Range;
import com.google.common.truth.Truth;
import se.digg.dgc.payload.v1.TestEntry;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.util.Objects;

import static org.junit.jupiter.api.Assertions.assertEquals;

public class LabResultTester {

    private final TestEntry entry;

    private LabResultTester(TestEntry entry) {
        this.entry = entry;
    }

    public static LabResultTester assertThatTestEntry(TestEntry entry) {
        return new LabResultTester(Objects.requireNonNull(entry));
    }

    public LabResultTester hasSampleTime(LocalDateTime sampleTime) {
        assertEquals(sampleTime.atZone(ZoneId.of("Europe/Copenhagen")).truncatedTo(ChronoUnit.SECONDS).toInstant(), entry.getSc().truncatedTo(ChronoUnit.SECONDS));
        return this;
    }

    public LabResultTester hasSampleTimeWithinRange(Range<LocalDateTime> range) {
        Truth.assertThat(LocalDateTime.ofInstant(entry.getSc(), ZoneId.of("Europe/Copenhagen"))).isIn(range);
        return this;
    }


    public LabResultTester isPcr() {
        assertEquals("LP6464-4", entry.getTt(), "Expected LOINC for PCR test");
        return this;
    }

    public LabResultTester isAG() {
        assertEquals("LP217198-3", entry.getTt(), "Expected LOINC for AG test");
        return this;
    }

    public LabResultTester isNegative() {
        assertEquals("260415000", entry.getTr(), "Expected 'detected negative' snomed code");
        return this;
    }

    public LabResultTester isPositive() {
        assertEquals("260373001", entry.getTr(), "Expected 'detected positive' snomed code");
        return this;
    }

    public LabResultTester hasManufacturer(String manufacturer) {
        assertEquals(manufacturer, entry.getMa());
        return this;
    }

    public LabResultTester hasTestCenter(String center) {
        assertEquals(center, entry.getTc());
        return this;
    }

}
