/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.util;

import com.google.common.collect.Range;
import com.google.common.truth.Truth;
import se.digg.dgc.payload.v1.RecoveryEntry;

import java.time.LocalDate;
import java.util.Objects;

import static org.junit.jupiter.api.Assertions.assertEquals;

public class RecoveryTester {

    private final RecoveryEntry entry;

    private RecoveryTester(RecoveryEntry entry) {
        this.entry = entry;
    }

    public static RecoveryTester assertThatRecoveryEntry(RecoveryEntry entry) {
        return new RecoveryTester(Objects.requireNonNull(entry));
    }

    public RecoveryTester hasSampleDate(LocalDate sampleDate) {
        assertEquals(sampleDate, entry.getFr());
        return this;
    }

    public RecoveryTester hasSampleDateWithinRange(Range<LocalDate> range) {
        Truth.assertThat(entry.getFr()).isIn(range);
        return this;
    }

    public RecoveryTester hasStartDate(LocalDate endDate) {
        assertEquals(endDate, entry.getDf());
        return this;
    }


    public RecoveryTester hasEndDate(LocalDate endDate) {
        assertEquals(endDate, entry.getDu());
        return this;
    }

    public RecoveryTester hasEndDateWithinRange(Range<LocalDate> range) {
        Truth.assertThat(entry.getDu()).isIn(range);
        return this;
    }
}
