/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.util;

import se.digg.dgc.payload.v1.VaccinationEntry;

import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;

import static org.junit.jupiter.api.Assertions.assertAll;
import static org.junit.jupiter.api.Assertions.assertEquals;

public class VaccinationTester {

    private final VaccinationEntry entry;

    private VaccinationTester(VaccinationEntry entry) {
        this.entry = entry;
    }

    public static VaccinationTester assertThatVaccination(VaccinationEntry entry) {
        return new VaccinationTester(entry);
    }


    public VaccinationTester hasDateOfVaccination(Instant effectuationTime) {
        assertEquals(effectuationTime.atZone(ZoneId.of("Europe/Copenhagen")).toLocalDate(), entry.getDt());
        return this;
    }

    public VaccinationTester hasDateOfVaccination(LocalDate effectuationDate) {
        assertEquals(effectuationDate, entry.getDt());
        return this;
    }

    public VaccinationTester isDoseOf(int doseNumber, int dosesInSeries) {
        assertAll(
            () -> assertEquals(doseNumber, entry.getDn(), "Unexpected dose number"),
            () -> assertEquals(dosesInSeries, entry.getSd(), "Unexpected doses in series")
        );
        return this;
    }

    public VaccinationTester hasCountryCode(String countryCode) {
        assertEquals(countryCode, entry.getCo());
        return this;
    }


    public VaccinationTester isAstraZenicaVaccination() {
        assertEquals("EU/1/21/1529", entry.getMp());
        return this;
    }

    public VaccinationTester isJohnsonVaccination() {
        assertEquals("EU/1/20/1525", entry.getMp());
        return this;
    }

    public VaccinationTester isModernaVaccination() {
        assertEquals("EU/1/20/1507", entry.getMp());
        return this;
    }

    public VaccinationTester isPfizerVaccination() {
        assertEquals("EU/1/20/1528", entry.getMp());
        return this;
    }
}
