/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.webservice.idws;

import java.util.Arrays;

public enum IdwsUser {

    CPR_2401010010("2401010010", "PID:9208-2002-2-337576502433", "Trine Jansen"),
    CPR_2401010011("2401010011", "PID:9208-2002-2-878306162689", "Jakob Jønsson Møller"),
    CPR_2401010012("2401010012", "PID:9208-2002-2-568409606447", "Tinja Gregersen"),
    CPR_2401010013("2401010013", "PID:9208-2002-2-257506556945", "Theiss Nielsen"),
    CPR_2401010014("2401010014", "PID:9208-2002-2-502257251507", "Tashina Mathiesen"),
    CPR_2401010015("2401010015", "PID:9208-2002-2-831965872884", "Thorborg Eriksen"),
    CPR_2401010016("2401010016", "PID:9208-2002-2-494976198983", "Thia Christoffersen"),
    EXPIRED_IMMUNITY("2401010017", "PID:9208-2002-2-604270903016", "Truels Holm"),
    CPR_2401010018("2401010018", "PID:9208-2002-2-652156092370", "Helle Frost Rasmussen"),
    NC_CPR_0405870101("0405870101", "PID:9208-2002-2-581917085017", "Anders Andersen", "CPAS-01", false), //Vaccinated 2 dosages (completed 24/4/2021), negative test & immunity + has child cpr '3008198989'
    NC_CPR_0405870104("0405870104", "PID:9208-2002-2-093262756303", "Dorthe Drewes", "CPAS-04", false),
    NC_CPR_0405870106("0405870106", "PID:9208-2002-2-063953863389", "Frederikke Frederiksen", "CPAS-06", false), //has only a negative (not EU approved) antigen test max 28 hours old + has child cpr '3008198989'
    NC_CPR_2011451636("2011451636", "PID:9208-2002-2-169261938102", "Åge Ågesen", "CPAS-34", false), //only negative PCR max 28 hours old
    NC_CPR_1704080278("1704080278", "PID:9208-2002-2-404223886309", "Tämirä Jäkøbsén", "CPAS-35", false), //positive PCR -4 hours from 10:00 DK time
    NC_CPR_2310901373("2310901373", "PID:9208-2002-2-450908862280", "Æske Æskesen", "CPAS-37", false), //positive AG -4 hours from 10:00 DK time
    NC_CPR_2310901374("2310901374", "PID:9208-2002-2-917848732737", "Thóra Thórasen", "CPAS-38", false), //has only a positive antigen test max 28 hours old
    NC_CPR_1204933477("1204933477", "PID:9208-2002-2-798738532325", "Torgny Madsen", "CPAS-43", false),
    NC_CPR_2910781588("2910781588", "PID:9208-2002-2-780815117191", "Trinke Larsen", "CPAS-44", false), //has completed vaccination (24/4/2021) + immunity via positive PCR
    NC_CPR_2306654281("2306654281", "PID:9208-2002-2-004919215067", "Tolver Sørensen", "CPAS-45", false), //has completed vaccination (24/4/2021) + immunity via positive AG
    NC_CPR_1102962840("1102962840", "PID:9208-2002-2-141845728817", "Ronja Lind", "CPAS-855", false), //has completed vaccination (24/4/2021) + revaccination (15/9/2021) + positive AG -336 hours
    NC_CPR_1206103647("1206103647", "PID:9208-2002-2-532630289941", "Sara Thorsen", "CPAS-520", false), //has completed vaccination (24/4/2021) + booster + positive AG -347 hours
    NC_CPR_1010742357("1010742357", "PID:9208-2002-2-808993368098", "Teis Andreasen", "CPAS-47", false), //PCR/PCR Påvist/Påvist	-288 time/-1 time
    NC_CPR_3107811778("3107811778", "PID:9208-2002-2-516883316645", "Telena Brandt", "CPAS-49", false), //positive AG -336 timer
    NC_CPR_0212442684("0212442684", "PID:9208-2002-2-623647797388", "Tinea Pedersen", "CPAS-76", false), //has only a positive pcr test min 14 days old
    NC_CPR_2104262681("2104262681", "PID:9208-2002-2-719169071905", "Lady Gaga", "CPAS-104", false),
    NC_CPR_2405125656("2405125656", "", "John Johnsen", "Barn af CPAS-104", false),
    NC_CPR_0107066666("0107066666", "", "Albert Albertsen", "Barn af CPAS-104", false), //15 år (og dermed ikke med i stamdata_child_relation)
    NC_CPR_0510740700("0510740700", "PID:9208-2002-2-654676179409", "Terna Ray Ray Bak", "CPAS-122", false), //has different addresseringsnavn + has 1/2 phizer (01/06/2021) + positive PCR min 333 hours old,
    NC_CPR_0501942071("0501942071", "PID:9208-2002-2-623181850198", "Torres Schou", "CPAS-123", false),
    NC_CPR_2705230254("2705230254", "PID:9208-2002-2-129060039582", "Sofus Emil Kragh Nielsen", "CPAS-208", false), //5 AG with manufacturer (CE / CE / unknown / HSC / HSC), taken (-160 / -136 / -112 / -88 / -12 hours from 10:00 DK time)
    NC_CPR_2009171897("2009171897", "PID:9208-2002-2-699748799280", "Amélie Babineaux", "CPAS-211", false), //has only negative (EU approved) antigen max 28 hours old
    NC_CPR_2305372528("2305372528", "PID:9208-2002-2-777335363829", "Eugène Hamilton Lévesque", "CPAS-214", false),
    NC_CPR_1105692051("1105692051", "PID:9208-2002-2-495323869520", "Øjvind Ølgård", "CPAS-215", false), //5 negative tests with type (PCR / CE / PCR / HSC / HSC), taken  (-160 / -136 / -112 / -88 / -12 hours from 10:00 DK time),
    NC_CPR_1511212961("1511212961", "PID:9208-2002-2-448494187394", "Niels Haakon Markussen", "CPAS-217", false), //positive PCR -263 hours from 10:00 DK time
    NC_CPR_1202363235("1202363235", "PID:9208-2002-2-923337994216", "Thord Karlsen", "CPAS-218", false),
    NC_CPR_1308620712("1308620712", "PID:9208-2002-2-737186069295", "Tanna Christensen", "CPAS-219", false), //positive pcr -4318 hours from 10:00 DK time,
    NC_CPR_2211282360("2211282360", "PID:9208-2002-2-281764867675", "Anders And", "CPAS-220", false), //negative PCR / AG  -73 / -72 hours from 10:00 DK time
    NC_CPR_3001333002("3001333002", "PID:9208-2002-2-853029834607", "Tami Dahl", "CPAS-222", false), //expired negative PCR -96 hours from 10:00 DK time
    NC_CPR_0208193805("0208193805", "PID:9208-2002-2-816833354897", "Terje Tøke Davidsen", "CPAS-234", false), //expired positive PCR more than 12 months old
    NC_CPR_2109624288("2109624288", "PID:9208-2002-2-377714663018", "Tordis Ravn", "CPAS-240", false), // HSC ikke påvist -4 timer
    NC_CPR_2302471279("2302471279", "PID:9208-2002-2-759747196454", "Thorbjørn Nielsen", "CPAS-241", false), //ingen tests, færdigvaccineret 01.06.21 kl.12
    NC_CPR_2906087879("2906087879", "", "Hans Nielsen", "Barn af CPAS-240 og CPAS-241", false),
    NC_CPR_3006077879("3006077879", "", "Hans Jørgen Nielsen", "Barn af CPAS-240 og CPAS-241", false),
    NC_CPR_0106107878("0106107878", "", "Hansine Nielsen", "Barn af CPAS-240 og CPAS-241", false), //PCR (Ikke påvist) / HSC (Ikke påvist)	 -97 timer / -4 timer
    NC_CPR_0611253169("0611253169", "PID:9208-2002-2-489359045050", "Terkild Lassen", "CPAS-245", false), //two positive PCRs, -384 timer / 528 timer
    NC_CPR_2104513363("2104513363", "PID:9208-2002-2-681904628606", "Thomas Clausen", "Unknown", false), //two positive PCRs,  -347 timer / -528 timer. See https://nsp-dk.slack.com/archives/C01RRS1HN0M/p1636457619037700
    NC_CPR_1801293574("1801293574", "PID:9208-2002-2-440602698067", "Sofia Poulsen", "CPAS-854", false), //a positive PCR -347 timer
    NC_CPR_3012283023("3012283023", "PID:9208-2002-2-217863913161", "Tarben Sørensen", "CPAS-263", false), //PCR / PCR / PCR / HSC / HSC / unknown || Ikke påvist / Påvist / Ikke påvist / Ikke påvist / Påvist / Ikke påvist || -4 / -8 / -12 / -16 / -20 / -24 timer
    NC_CPR_2506893339("2506893339", "PID:9208-2002-2-063497001167", "Thorlejv Knudsen", "CPAS-264", false), //AG / PCR || Påvist / Påvist || -4 / -1440
    NC_CPR_0706293072("0706293072", "PID:9208-2002-2-203578441531", "Torunn Kristoffersen", "CPAS-265", false),
    NC_CPR_0712081266("0712081266", "PID:9208-2002-2-758063672791", "Thilla Hermansen", "CPAS-266", false),
    NC_CPR_1703501676("1703501676", "PID:9208-2002-2-255528372885", "Feline Jepsen", "CPAS-850", false), //En negativ PCR test -107 timer
    NC_CPR_1301071035("1301071035", "PID:9208-2002-2-445627260913", "Mads Danielsen", "CPAS-851", false), //En positiv PCR test -275 timer
    NC_CPR_1403993158("1403993158", "PID:9208-2002-2-869862581500", "Tine Kjeldsen", "CPAS-852", false), //positiv PCR -4331 timer
    NC_CPR_0301301728("0301301728", "PID:9208-2002-2-358488325199", "Katinka Sørensen", "CPAS-853", false), //user has 5 negative AG with manufacturer (CE / CE / unknown / HSC / HSC), taken (-100 / -90 / -80 / -70 / -60 hours from 10:00 DK time)
    NC_CPR_0811220950("0811220950", "PID:9208-2002-2-843389725503", "Hansine Dam", "CPAS-856", false), //En positiv PCR test -347 timer+vaccination

    NO_LAB_RESULTS("1505920607", "PID:9208-2002-2-661016392995", "Thinus Kjeldsen"),
    IN_NSP_STAMDATA_BUT_NOT_IN_CPR("0806644420", "PID:9208-2002-2-124383634885", "Tara Skov"),
    NOT_IN_NSP_STAMDATA("3005027911", "PID:9208-2002-2-202096693317", "Ticho Schultz"),
    POWER_OF_ATTORNEY_ACTING_USER("3108732076", "PID:9208-2002-2-098076399641", "Gitte Gregersen"), //har fuldmagt fra 0904983500 Gry Gregersen
    POWER_OF_ATTORNEY_PRINCIPAL_USER("0904983500", "", "Gry Gregersen"), //har givet fuldmagt til 3108732076 Gitte Gregersen

    GYLDIGT_IMMUNITET_PAS("0101859568", "PID:9208-2002-2-615336165630", "Ture Toft"),          //(gyldig immunitet)
    PARENT_A("0601830299", "PID:9208-2002-2-954285144302", "Thorvil Jespersen"),
    CHILD_OF_A("1801137268", "", "Barn af Thorvil Jespersen"),

    // the names on these 4 users must not be changed
    PARENT_B("2502801010", "PID:9208-2002-2-584482341320", "Anna Andersen"),
    CHILD_1_OF_B("2503107875", "", "Anton Andersen"),
    CHILD_2_OF_B("0706128662", "", "Nadja Andersen"),
    CHILD_3_OF_B("1005189655", "", "Kalle Andersen");


    private final String name;
    private final String alias;
    private final String cpr;
    private final String pid;
    private final boolean allowDdvChanges;

    IdwsUser(String cpr, String pid, String name) {
        this.name = name;
        this.cpr = cpr;
        this.pid = pid;
        this.alias = null;
        this.allowDdvChanges = true;
    }

    IdwsUser(String cpr, String pid, String name, String alias, boolean allowDdvChanges) {
        this.name = name;
        this.alias = alias;
        this.cpr = cpr;
        this.pid = pid;
        this.allowDdvChanges = allowDdvChanges;
    }

    public static IdwsUser fromName(String name) {
        return Arrays.stream(values())
            .filter(user -> user.commonName().equals(name))
            .findFirst()
            .orElse(null);
    }

    public String certificateInfo() {
        return "SubjectDN={C=DK,O=Ingen organisatorisk tilknytning,CN=" + name + ",Serial=" + pid + "},IssuerDN={C=ignored},CertSerial={1234}";
    }

    public String commonName() {
        return name;
    }

    public String cpr() {
        return cpr;
    }

    public String pid() {
        return pid;
    }

    public boolean allowDdvChanges() {
        return allowDdvChanges;
    }
}
