/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.kids.dao;

import com.fasterxml.jackson.annotation.JsonGetter;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonSetter;
import org.springframework.lang.NonNull;

import java.time.LocalDateTime;

/**
 * Simple implementation of {@link KidsLabResult}
 */
public class KidsLabResultImpl implements KidsLabResult {
    private static final String CPR_PROPERTY = "cpr";
    private static final String CENTER_PROPERTY = "center";
    private static final String CREATION_TIME_PROPERTY = "creationTime";
    private static final String ID_PROPERTY = "id";
    private static final String MANUFACTURER_PROPERTY = "manufacturer";
    private static final String NAME_PROPERTY = "name";
    private static final String RESULT_PROPERTY = "result";
    private static final String RESULT_TIME_PROPERTY = "resultTime";
    private static final String SAMPLE_TIME_PROPERTY = "sampleTime";
    private static final String TYPE_PROPERTY = "type";

    static final String CENTER_COLUMN_LABEL = "center";
    static final String CPR_COLUMN_LABEL = "cpr";
    static final String CREATION_TIME_COLUMN_LABEL = "creation_timestamp";
    static final String ID_COLUMN_LABEL = "id";
    static final String MANUFACTURER_COLUMN_LABEL = "manufacturer";
    static final String NAME_COLUMN_LABEL = "name";
    static final String RESULT_COLUMN_LABEL = "result";
    static final String RESULT_TIME_COLUMN_LABEL = "result_timestamp";
    static final String SAMPLE_TIME_COLUMN_LABEL = "sample_timestamp";
    static final String TYPE_COLUMN_LABEL = "type";

    private String center;
    private String cpr;
    private LocalDateTime creationTime;
    private long id;
    private String manufacturer;
    private String name;
    private String result;
    private LocalDateTime resultTime;
    private LocalDateTime sampleTime;
    private String type;

    KidsLabResultImpl() {
    }

    @NonNull
    @Override
    @JsonGetter
    @JsonProperty(CENTER_PROPERTY)
    public String center() {
        return center;
    }

    @JsonSetter
    @JsonProperty(CENTER_PROPERTY)
    public KidsLabResultImpl center(@NonNull final String center) {
        this.center = center;
        return this;
    }

    @NonNull
    @Override
    @JsonGetter
    @JsonProperty(CPR_PROPERTY)
    public String cpr() {
        return cpr;
    }

    @JsonSetter
    @JsonProperty(CPR_PROPERTY)
    public KidsLabResultImpl cpr(@NonNull final String cpr) {
        this.cpr = cpr;
        return this;
    }

    @NonNull
    @Override
    @JsonGetter
    @JsonProperty(CREATION_TIME_PROPERTY)
    public LocalDateTime creationTime() {
        return creationTime;
    }

    @JsonSetter
    @JsonProperty(CREATION_TIME_PROPERTY)
    public KidsLabResultImpl creationTime(@NonNull final LocalDateTime creationTime) {
        this.creationTime = creationTime;
        return this;
    }

    @Override
    @JsonGetter
    @JsonProperty(ID_PROPERTY)
    public long id() {
        return id;
    }

    @JsonSetter
    @JsonProperty(ID_PROPERTY)
    public KidsLabResultImpl id(final long id) {
        this.id = id;
        return this;
    }

    @NonNull
    @Override
    @JsonGetter
    @JsonProperty(MANUFACTURER_PROPERTY)
    public String manufacturer() {
        return manufacturer;
    }

    @JsonSetter
    @JsonProperty(MANUFACTURER_PROPERTY)
    public KidsLabResultImpl manufacturer(@NonNull final String manufacturer) {
        this.manufacturer = manufacturer;
        return this;
    }

    @NonNull
    @Override
    @JsonGetter
    @JsonProperty(NAME_PROPERTY)
    public String name() {
        return name;
    }

    @JsonSetter
    @JsonProperty(NAME_PROPERTY)
    public KidsLabResultImpl name(@NonNull String name) {
        this.name = name;
        return this;
    }

    @NonNull
    @Override
    @JsonGetter
    @JsonProperty(RESULT_PROPERTY)
    public String result() {
        return result;
    }

    @JsonSetter
    @JsonProperty(RESULT_PROPERTY)
    public KidsLabResultImpl result(@NonNull final String result) {
        this.result = result;
        return this;
    }

    @NonNull
    @Override
    @JsonGetter
    @JsonProperty(RESULT_TIME_PROPERTY)
    public LocalDateTime resultTime() {
        return resultTime;
    }

    @JsonSetter
    @JsonProperty(RESULT_TIME_PROPERTY)
    public KidsLabResultImpl resultTime(@NonNull final LocalDateTime resultTime) {
        this.resultTime = resultTime;
        return this;
    }

    @NonNull
    @Override
    @JsonGetter
    @JsonProperty(SAMPLE_TIME_PROPERTY)
    public LocalDateTime sampleTime() {
        return sampleTime;
    }

    @JsonSetter
    @JsonProperty(SAMPLE_TIME_PROPERTY)
    public KidsLabResultImpl sampleTime(@NonNull final LocalDateTime sampleTime) {
        this.sampleTime = sampleTime;
        return this;
    }

    @NonNull
    @Override
    @JsonGetter
    @JsonProperty(TYPE_PROPERTY)
    public String type() {
        return type;
    }

    @JsonSetter
    @JsonProperty(TYPE_PROPERTY)
    public KidsLabResultImpl type(@NonNull final String type) {
        this.type = type;
        return this;
    }
}
